/// <reference path="../functions.jsx" />

function autumnEffect() {

    var document = app.activeDocument;
    var undoAction = markHistoryPosition();

    try {

        Beautypanel.checkBitsPerChannel();
        Beautypanel.checkBackgroundLayer();
        Beautypanel.refreshLayers();
        var useMask = true;
        var deleteAndContinue = false;

        // ============================================== //
        // If layer exists, ask user for permission to delete it.
        if ($.layers.autumn) {
            deleteAndContinue = confirm(strings.request_undo_autumn);
        }

        // ============================================== //
        if (!$.layers.autumn || deleteAndContinue) {
            getReferenceLayer();

            // ============================================== //
            var autumnMaskLayer = createAdjustmentLayer();

            // ============================================== //
            // Decide to use mask or layer for result
            if (useMask) {
                $.layers.autumn.remove();
                $.layers.autumn = autumnMaskLayer;
            } else {
                $.layers.autumn = autumnMaskLayer.merge();
            }

            // ============================================== //
            // Finalize layer
            $.layers.autumn.name = strings.layername_autumn;
            $.layers.autumn.opacity = 50;

        }

        $.layers.autumn.move(document.artLayers[0], ElementPlacement.PLACEBEFORE);

    } catch (err) {
        undoAction();
        alert("Error at 'Autumn Effect.jsx': " + err);
    }

    function getReferenceLayer() {
        // ============================================== //
        // If layer already exists. Delete it.
        if ($.layers.autumn) {
            $.layers.autumn.remove();
        }
        // ============================================== //
        // Create the reference layer
        if ($.layers.detail && $.layers.soft) {
            // ============================================== //
            // Document has frequency-separation $.layers.
            // Create a merged layer from Detail and Soft layer.
            // Use it as reference layer.
            if ($.layers.levels) {
                $.layers.levels.visible = false;
            }
            $.layers.soft.visible = true;
            $.layers.detail.visible = true;
            $.layers.autumn = document.mergeLayers($.layers.detail.duplicate(), $.layers.soft.duplicate());
        } else {
            // Create a copy of background layer and use it as reference layer
            $.layers.autumn = document.activeLayer.duplicate();
        }
        // ============================================== //
        // Set default attributes
        $.layers.autumn.blendMode = BlendMode.NORMAL;
        $.layers.autumn.opacity = 100;
        $.layers.autumn.name = strings.layername_autumn;
        // ============================================== //
        // Set reference layer as selected layer
        document.activeLayer = $.layers.autumn;
        // ============================================== //
        return $.layers.autumn;
    }

    function createAdjustmentLayer(withDialog) {
        // ============================================== //
        var dialogMode = (withDialog ? DialogModes.ALL : DialogModes.NO);
        var desc1 = new ActionDescriptor();
        var ref = new ActionReference();
        ref.putClass(cTID('AdjL'));
        desc1.putReference(cTID('null'), ref);
        var desc2 = new ActionDescriptor();
        var desc3 = new ActionDescriptor();
        desc3.putEnumerated(sTID("presetKind"), sTID("presetKindType"), sTID("presetKindDefault"));
        desc3.putBoolean(cTID('Clrz'), false);
        desc2.putObject(cTID('Type'), cTID('HStr'), desc3);
        desc1.putObject(cTID('Usng'), cTID('AdjL'), desc2);
        executeAction(cTID('Mk  '), desc1, dialogMode);
        // ============================================== //
        setAdjustmentLayerValues(withDialog);
        // ============================================== //
        return document.activeLayer;
    }

    function setAdjustmentLayerValues(withDialog) {
        var dialogMode = (withDialog ? DialogModes.ALL : DialogModes.NO);
        var desc1 = new ActionDescriptor();
        var ref1 = new ActionReference();
        ref1.putEnumerated(cTID('AdjL'), cTID('Ordn'), cTID('Trgt'));
        desc1.putReference(cTID('null'), ref1);
        var desc2 = new ActionDescriptor();
        desc2.putEnumerated(sTID("presetKind"), sTID("presetKindType"), sTID("presetKindCustom"));
        var list1 = new ActionList();
        var desc3 = new ActionDescriptor();
        desc3.putInteger(cTID('LclR'), 3);
        desc3.putInteger(cTID('BgnR'), 24);
        desc3.putInteger(cTID('BgnS'), 105);
        desc3.putInteger(cTID('EndS'), 135);
        desc3.putInteger(cTID('EndR'), 195);
        desc3.putInteger(cTID('H   '), -95);
        desc3.putInteger(cTID('Strt'), 0);
        desc3.putInteger(cTID('Lght'), 0);
        list1.putObject(cTID('Hst2'), desc3);
        desc2.putList(cTID('Adjs'), list1);
        desc1.putObject(cTID('T   '), cTID('HStr'), desc2);
        executeAction(cTID('setd'), desc1, dialogMode);
    }

}

autumnEffect();
