/// <reference path="../functions.jsx" />

#include "../prototype/Document.jsx"
#include "../extends/LayerUtils.jsx"

function applyOrtonEffect() {

    var undoAction = markHistoryPosition();

    try {

        Beautypanel.checkBitsPerChannel();
        Beautypanel.checkBackgroundLayer();
        Beautypanel.refreshLayers();

        // ===================================================================== //
        var document = app.activeDocument;

        if (!document) {
            return;
        }

        // ===================================================================== //
        var backgroundLayer = document.backgroundLayer;

        // ===================================================================== //
        // Remove layer if exists
        if (layers.orthon) {
            layers.orthon.remove();
            layers.orthon = null;
        }

        // ===================================================================== //
        // Create Orton layer
        if (layers.soft && layers.detail) {

            // ===================================================================== //
            // If Soft and Detail layers exist, use them to generate the orton layer

            if (layers.levels) {
                layers.levels.visible = false;
            }

            backgroundLayer.visible = false;
            layers.soft.visible = true;
            layers.detail.visible = true;

            layers.orthon = document.mergeLayers(layers.detail.duplicate(), layers.soft.duplicate());

        } else {

            // ===================================================================== //
            // If no Soft and Detail layers exist, create one from background layer
            $.layers.orthon = backgroundLayer.duplicate();

        }

        // ===================================================================== //
        // set name, blendMode and opacity of orton layer
        $.layers.orthon.name = strings.layername_orthon;
        $.layers.orthon.blendMode = BlendMode.NORMAL;
        $.layers.orthon.opacity = 50;

        // ===================================================================== //
        // Set orton layer as active layer
        document.activeLayer = $.layers.orthon;

        // ===================================================================== //
        // Open dialog to apply a gaussian blur
        usingDialogMode(DialogModes.ALL, function () {
            $.layers.orthon.applyGaussianBlur(6, 5);
        });

        // ===================================================================== //
        // Create a new rvla mask on orton layer
        $.LayerUtils.createRvlaMask($.layers.orthon);

        $.layers.orthon.move(document.artLayers[0], ElementPlacement.PLACEBEFORE);

    } catch (err) {
        alert("Orthon Effect 2: " + err.message);
        undoAction();
    }
}

applyOrtonEffect();
