/// <reference path="../functions.jsx" />

function vignette() {

    // === Constants

    var VignetteMode = {
        NORMAL: 0,
        PREVIEW: 1
    }

    var undoAction = markHistoryPosition();

    try {

        Beautypanel.checkBitsPerChannel();
        Beautypanel.checkBackgroundLayer();
        Beautypanel.refreshLayers();

        var document = app.activeDocument;

        var config = BEAUTYPANEL.config.vignette;
        var withDialog = true;
        var opacity = 25;

        // ===================================================================== //
        // Check the execution mode (NORMAL|PREVIEW)
        if (config.mode == VignetteMode.NORMAL) {
            if ($.layers.vignette) {
                var deleteAndContinue = confirm(strings.request_undo_vignette);
                if (!deleteAndContinue) {
                    return;
                }
            }
        } else {
            withDialog = false;
        }

        // ===================================================================== //
        // Delete existing layer
        if ($.layers.vignette) {
            opacity = $.layers.vignette.opacity;
            $.layers.vignette.remove();
        }

        // ===================================================================== //
        // Create vignette layer
        $.layers.vignette = document.artLayers.add();
        $.layers.vignette.name = strings.layername_vignette;

        // ===================================================================== //
        fillLayer(100);
        createElipse(config.border);
        deleteSelected();
        disableSelection();
        openGaussianBlurDialog(config.blurRadius, withDialog);

        // ===================================================================== //
        // Finalize
        $.layers.vignette.opacity = opacity;
        $.layers.vignette.move(document.artLayers[0], ElementPlacement.PLACEBEFORE);

    } catch (err) {
        undoAction();
        return [
            "error",
            err
        ]
    }


    function fillLayer(opacity) {
        var desc198 = new ActionDescriptor();
        desc198.putEnumerated(Codes.using, Codes.fillContents, Codes.black);
        desc198.putUnitDouble(Codes.opacity, Codes.percentUnit, opacity);
        desc198.putEnumerated(Codes.mode, Codes.blendMode, Codes.normal);
        executeAction(Codes.fill, desc198, DialogModes.NO);
    }

    function createElipse(padding) {

        padding = padding / 100;

        var width = app.activeDocument.width.as("px");
        var height = app.activeDocument.height.as("px");

        var top = height * padding;
        var left = width * padding;
        var bottom = height * (1.0 - padding);
        var right = width * (1.0 - padding);

        if (BEAUTYPANEL.config.vignette.type === "circle") {

            var size = Math.min(width, height);

            // Duplicate padding for both edges
            padding *= 2;
            padding = size * padding;

            // Calculate diameter and radius for circle
            var diameter = size - padding;
            var radius = diameter * 0.5;

            top = height * 0.5 - radius;
            left = width * 0.5 - radius;
            right = width * 0.5 + radius;
            bottom = height * 0.5 + radius;

        }

        var idsetd = charIDToTypeID("setd");
        var desc203 = new ActionDescriptor();
        var idnull = charIDToTypeID("null");
        var ref53 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idfsel = charIDToTypeID("fsel");
        ref53.putProperty(idChnl, idfsel);
        desc203.putReference(idnull, ref53);

        var idT = charIDToTypeID("T   ");
        var desc204 = new ActionDescriptor();

        var idTop = charIDToTypeID("Top ");
        var idRlt = charIDToTypeID("#Pxl");
        desc204.putUnitDouble(idTop, idRlt, top);

        var idLeft = charIDToTypeID("Left");
        var idRlt = charIDToTypeID("#Pxl");
        desc204.putUnitDouble(idLeft, idRlt, left);

        var idBtom = charIDToTypeID("Btom");
        var idRlt = charIDToTypeID("#Pxl");
        desc204.putUnitDouble(idBtom, idRlt, bottom);

        var idRght = charIDToTypeID("Rght");
        var idRlt = charIDToTypeID("#Pxl");
        desc204.putUnitDouble(idRght, idRlt, right);

        var idElps = charIDToTypeID("Elps");
        desc203.putObject(idT, idElps, desc204);
        var idAntA = charIDToTypeID("AntA");
        desc203.putBoolean(idAntA, true);
        executeAction(idsetd, desc203, DialogModes.NO);
    }

    function deleteSelected() {
        var idDlt = charIDToTypeID("Dlt ");
        executeAction(idDlt, undefined, DialogModes.NO);
    }

    function openGaussianBlurDialog(radius, withDialog) {
        var width = Math.max(app.activeDocument.width.as("px"), app.activeDocument.height.as("px"));
        var desc = new ActionDescriptor();
        desc.putUnitDouble(cTID("Rds "), cTID("#Pxl"), width * (radius / 100));
        executeAction(cTID("GsnB"), desc, withDialog ? DialogModes.ALL : DialogModes.ERROR);
    }

    function disableSelection() {
        var idsetd = charIDToTypeID("setd");
        var desc262 = new ActionDescriptor();
        var idnull = charIDToTypeID("null");
        var ref84 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idfsel = charIDToTypeID("fsel");
        ref84.putProperty(idChnl, idfsel);
        desc262.putReference(idnull, ref84);
        var idT = charIDToTypeID("T   ");
        var idOrdn = charIDToTypeID("Ordn");
        var idNone = charIDToTypeID("None");
        desc262.putEnumerated(idT, idOrdn, idNone);
        executeAction(idsetd, desc262, DialogModes.NO);
    }

}

vignette();
