/// <reference path="./functions.jsx" />

#include "./SuS.Photoshop.jsx"

function frequencySeparation() {

    var result = [];
    var undoAction = markHistoryPosition();
    var SuS_PS = SuS.Photoshop;

    try {

        Beautypanel.checkBitsPerChannel();
        Beautypanel.checkBackgroundLayer();
        Beautypanel.refreshLayers();

        setDPI(300);

        var document = app.activeDocument;

        var referenceLayer = document.backgroundLayer;
        var layersExist = false;
        var deleteAndContinue = false;
        var optionalDialogMode = BEAUTYPANEL.config.freqSep.withDialogs ? DialogModes.ALL : DialogModes.ERROR;

        if (Beautypanel.layers.referenceLayer != null) {
            referenceLayer = Beautypanel.layers.referenceLayer;
        }

        // ======================================================= //
        // Check existing layers and ask user for permission to delete them if required
        if (Beautypanel.layers.detail || Beautypanel.layers.soft || Beautypanel.layers.levels) {
            layersExist = true;
            deleteAndContinue = confirm(strings.request_undo_frequency_separation);
        }

        // ======================================================= //
        if (!layersExist || deleteAndContinue) {

            // ======================================================= //
            // Delete layers if they exist
            if (Beautypanel.layers.soft) 
                Beautypanel.layers.soft.remove();
            if (Beautypanel.layers.detail) 
                Beautypanel.layers.detail.remove();
            if (Beautypanel.layers.levels) 
                Beautypanel.layers.levels.remove();

            // ======================================================= //
            // Create layers Soft and Detail
            Beautypanel.layers.detail = referenceLayer.duplicate()
            Beautypanel.layers.detail.name = strings.layername_detail;

            Beautypanel.layers.soft = referenceLayer.duplicate();
            Beautypanel.layers.soft.name = strings.layername_soft;

            // ======================================================= //
            // Interpolate brightness
            usingDialogMode(optionalDialogMode, function () {
                Beautypanel.layers.soft.applyMedianNoise(10.0);
            });

            // ======================================================= //
            // Picture calculation
            Beautypanel.LayerUtils.applyCalculation(Beautypanel.layers.detail, Beautypanel.layers.soft, "RGB ", true, "Add ", 2, 0);
            Beautypanel.layers.detail.blendMode = BlendMode.LINEARLIGHT;

            // ======================================================= //
            // Create adjustment layer (levels)
            Beautypanel.layers.levels = Beautypanel.LayerUtils.createContrastLayer(Beautypanel.layers.detail, 120, 132);
            Beautypanel.layers.levels.name = strings.layername_levels;

            // ======================================================= //
            // Update layers visibility
            referenceLayer.visible = false;
            Beautypanel.layers.soft.visible = false;

            // ======================================================= //
            // Auf Ebene Detail wechseln
            document.activeLayer = Beautypanel.layers.detail;
            
            finishFrequencySeparation();

        }

    } catch (err) {
        alert("Error in 'Frequency Separation.jsx': " + err)
        undoAction();
    }

    return result.join('|');
}

/**
 * Select cloneStampTool and set opacity|hardness to 100%
 */
function finishFrequencySeparation() {
    try {

        var targetBrush = "cloneStampTool";

        // ======================================================= //
        // Kopierstempel auswählen und Preset aktivieren
        app.currentTool = targetBrush;
        

        // ======================================================= //
        // Get current brush diameter to set brush hardness
        // Somehow it is requried to apply a new hardness value
        var ref = new ActionReference();
        ref.putEnumerated(charIDToTypeID("capp"), charIDToTypeID("Ordn"), charIDToTypeID("Trgt"));
        var currentBrush = executeActionGet(ref).getObjectValue(stringIDToTypeID("currentToolOptions")).getObjectValue(charIDToTypeID("Brsh"));
        var diameter = currentBrush.getDouble(charIDToTypeID("Dmtr"));

        // ======================================================= //
        // Set Hardness to 100%
        var ref2 = new ActionReference();
        var desc19 = new ActionDescriptor();
        var desc = new ActionDescriptor();
        ref2.putEnumerated(charIDToTypeID("Brsh"), charIDToTypeID("Ordn"), charIDToTypeID("Trgt"));
        desc19.putReference(charIDToTypeID("null"), ref2);
        desc.putUnitDouble(charIDToTypeID("Dmtr"), charIDToTypeID("#Pxl"), diameter);
        desc.putUnitDouble(charIDToTypeID("Hrdn"), charIDToTypeID("#Prc"), 100);
        desc19.putObject(charIDToTypeID("T   "), charIDToTypeID("Brsh"), desc);
        executeAction(charIDToTypeID("setd"), desc19, DialogModes.ERROR);

        // ======================================================= //
        // Set Opacity to 100%
        //SuS_PS.setCurrentToolMainOptions({ opacity: 100 });
        var desc = new ActionDescriptor();
        var data = new ActionDescriptor();
        var ref = new ActionReference();
        ref.putClass(stringIDToTypeID(targetBrush));
        desc.putReference(Codes.null, ref);
        data.putUnitDouble(Codes.opacity, Codes.percentUnit, 100);
        desc.putObject(Codes.to, Codes.null, data);
        executeAction(Codes.set, desc, DialogModes.ERROR);

    } catch (err) {
        alert(err);
    }
}

frequencySeparation();
