/// <reference path="../functions.jsx" />

#include "../ConfigFile.jsx"

function saveCopy() {

	var wtf = null;
	var result = {
		type: null,
		mesage: null
	};

	var document;
	var originalDocument;
	var error = null;
	var makeCopy = true;
	var savedFile = null;

	var config = BEAUTYPANEL.config.save;

	try {

		// ======================================================= //
		// Open and read settings.wtf
		wtf = new ConfigFile(BEAUTYPANEL.files.settings);
		wtf.autoWrite = false;

		// ======================================================= //
		// Check if we clicked the "Save as" or the "Save as previous" button.
		// If we clicked "Save as previous", the save will run without any dialog.
		var overrideDialogMode = config.asPrevious ? DialogModes.ERROR : DialogModes.ALL;

		// ======================================================= //
		error = usingDialogMode(overrideDialogMode, function () {

			// ======================================================= //
			// Create a copy of current document
			// ======================================================= //

			originalDocument = app.activeDocument;
			document = app.activeDocument;
			if (makeCopy) {
				//document = originalDocument.duplicate();
				document = cloneDocument(originalDocument);
				setDPI(300);
			}

			// ======================================================= //
			// If we have detail and soft layer, set visibility
			// ======================================================= //

			Beautypanel.refreshLayers();
			if ($.layers.detail && $.layers.soft) {
				if ($.layers.levels) {
					$.layers.levels.visible = false;
				}
				$.layers.detail.visible = true;
				$.layers.soft.visible = true;
			}

			// ======================================================= //
			// Resize the copy
			// ======================================================= //

			if (config.scale) {

				// =======================================================
				// Read save_resolution and set new values for width and height.
				var newSize = UnitValue(wtf.getValue("save_resolution"), "px");
				var w = document.width.as("px");
				var h = document.height.as("px");
				var nW = (w > h)
					? newSize
					: undefined;
				var nH = (w > h)
					? undefined
					: newSize;

				// ======================================================= //
				// Resize the picture. It also opens the dialog if app.displayDialogs has value DialogModes.ALL
				// If an edge has value undefined, Photoshop will calculate it by picture's aspect ratio.
				document.resizeImage(nW, nH, null, ResampleMethod.BICUBIC)

				// ======================================================= //
				// The user could have changed the size within the dialog called by document.resizeImage. If it has changed, apply the new value to settings.wtf
				wtf.setValue("save_resolution", Math.max(document.width.as("px"), document.height.as("px")));

			}

			// ======================================================= //
			// Run command 'UnSharp Mask' on copy
			// ======================================================= //

			if (config.scale) {

				var usmRemove = app.activeDocument.activeHistoryState;

				var amount = parseFloat(wtf.getValue("save_unsharp_amount"));
				var radius = parseFloat(wtf.getValue("save_unsharp_radius"));
				var threshold = parseFloat(wtf.getValue("save_unsharp_threshold"));
				var withDialog = app.displayDialogs === DialogModes.ALL;

				// ======================================================= //
				// Create descriptor for UnSharpMask using parameters
				// Execute action and pick the returning descriptor.
				var descriptor = Beautypanel.unsharpMask(amount, radius, threshold, true, withDialog);

				var newAmount = descriptor.getDouble(Codes.amount);
				var newRadius = descriptor.getDouble(Codes.radius);
				var newThreshold = descriptor.getInteger(Codes.threshold);

				if (!withDialog) {
					// We do not open the unsharp mask dialog. So the resulted values should equal the input values
					if (amount !== newAmount || radius !== newRadius || threshold !== newThreshold) {
						// Unsharp mask bug occured
						if (confirm("Command 'UnSharp Mask' did not use the input values. This is a bug within photoshop. Should i revert the 'UnSharp Mask' command?")) {
							app.activeDocument.activeHistoryState = usmRemove;
						}
					}
				}

				// ======================================================= //
				// Write the submitted values into settings.wtf to use them next time as default.
				if (wtf && descriptor) {
					wtf.setValue("save_unsharp_amount", newAmount);
					wtf.setValue("save_unsharp_radius", newRadius);
					wtf.setValue("save_unsharp_threshold", newThreshold);
				}

			}

			// ======================================================= //
			// Save the copy to file
			// ======================================================= //

			var folder = "", name = "";
			try {
				folder = originalDocument.path;
				name = document.name;
			} catch (err) {
				// Document is not saved or Photoshop lost file bindings
			}

			// ======================================================= //
			// Open Dialog to select Path where the picture should be written.
			// ======================================================= //

			var targetPath = wtf.getValue("save_folder_" + config.path) || path;
			var format = wtf.getValue("save_format").toLowerCase();
			targetPath += "/" + removeFileExtension(name) + "." + format;

			// =========================================================================== //
			// Compare selected and original file path
			// If they equal, ask user for permission to overwrite the original document
			// =========================================================================== //

			if (app.displayDialogs !== DialogModes.ALL && targetPath === path + "/" + name) {
				var _continue = confirm(strings.message_overwrite_document);
				if (!_continue) {
					throw "Cancel";
				}
			}

			// ======================================================= //
			// Save the picture to selected file
			// ======================================================= //

			var actionDescriptor = new ActionDescriptor();
			var saveOptionsDescriptor = new ActionDescriptor();
			// Set Save Options
			var formatTypeId = Codes.JPEG;
			switch (format.toLowerCase()) {

				// Save as JPEG
				case "jpg":
				case "jpeg":
				case "jpe":
					formatTypeId = Codes.JPEG;
					saveOptionsDescriptor.putInteger(Codes.extendedQuality, 11);
					saveOptionsDescriptor.putEnumerated(Codes.matteColor, Codes.matteColor, Codes.none);
					break;

				// Save as GIF
				case "gif":
					formatTypeId = Codes.GIFFormat;
					saveOptionsDescriptor.putInteger(Codes.colors, 256)
					break;

				// Save as PNG
				case "png":
				case "pns":
					formatTypeId = Codes.PNGFormat;
					saveOptionsDescriptor.putInteger(Codes.compression, 9);
					break;

				// Save as TIFF
				case "tiff":
				case "tif":
					formatTypeId = Codes.TIFFFormat;
					break;

				// Save as PSB
				case "psb":
					formatTypeId = Codes.largeDocumentFormat;
					break;

				// Save as Photoshop PSD
				case "psd":
					formatTypeId = Codes.photoshop35Format;
					break;

				// Save as Bump
				case "bmp":
				case "rle":
				case "dib":
					formatTypeId = Codes.bMPFormat;
					saveOptionsDescriptor.putBoolean(Codes.compression, true);
					break;
			}
			actionDescriptor.putObject(Codes.as, formatTypeId, saveOptionsDescriptor);
			// End Set Save Options
			actionDescriptor.putPath(Codes.in, new File(targetPath));
			actionDescriptor.putEnumerated(Codes.saveStage, Codes.saveStageType, Codes.saveSucceeded);
			var resultDescriptor = executeAction(Codes.save, actionDescriptor, app.displayDialogs);

			// ======================================================= //
			// Update config with data from result descriptor
			// ======================================================= //

			savedFile = resultDescriptor.getPath(Codes.in);
			wtf.setValue("save_format", /(?:\.([^.]+))?$/.exec(savedFile.name)[1]);
			wtf.setValue("save_folder_" + config.path, savedFile.path);

		}, true);

		// ======================================================= //
		if (error) {
			throw error;
		}

		// ======================================================= //
		// Save changes to settings.wtf
		// ======================================================= //

		if (wtf) {
			wtf.write();
		}

		result.type = "success";
		result.message = 'Saved to "' + savedFile.fullName + '"';

	} catch (err) {
		if (err === "abort") {
			result.type = "abort";
		} else {
			result.type = "error"
			result.message = err;
			alert(err);
		}
	} finally {
		// ======================================================= //
		// Close the duplicated document
		// ======================================================= //
		if (document !== originalDocument) {
			document.close(SaveOptions.DONOTSAVECHANGES);
		}
	}
	return JSON.stringify(result);
}

function cloneDocument(sourceDocument) {

	var cloneDocument,
		clonename = "",
		i = 1;

	clonename = sourceDocument.name + " " + i;

	while (documentNameInUse(clonename)) {
		i++;
		clonename = sourceDocument.name + " " + i;
	}

	usingDialogMode(DialogModes.ERROR, function () {
		cloneDocument = sourceDocument.duplicate(clonename);
	})

	// alert(
	// 	"sourceDocument.name: '" + sourceDocument.name + "'\n"
	// 	+ "cloneDocument.name: '" + cloneDocument.name + "'"
	// );

	if (cloneDocument.name === sourceDocument.name) {
		alert("Duplicate document name equals source document name.");
	}

	return cloneDocument;
}

function documentNameInUse(name) {
	for (var i = 0; i < app.documents.length; i++) {
		if (app.documents[i].name === name) {
			return true;
		}
	}
	return false;
}

saveCopy();
