/// <reference path="../functions.jsx" />

function maskedSharpen() {

    var undoAction = markHistoryPosition();

    var RecreateLevelsSettingsLayer = false;

    try {

        Beautypanel.refreshLayers();

        // Ausgangslayer
        var layer = getReferenceLayer();
        layer.name = strings.layername_maskedSharpen;

        app.activeDocument.activeLayer = layer;

        // ===================================================================== //
        // Smartobjekt Konvertierung
        convertLayerToSmartobject(layer);

        // ===================================================================== //
        // Konvertierung in LAB Modus
        convertToLABMode();

        // ===================================================================== //
        // Helligkeitskanal duplizieren
        duplicateBrightnessChannel(layer);

        // ===================================================================== //
        // Konturen finden
        findContours(layer);

        // ===================================================================== //
        // Konturlinien Einstellen
        levels(layer);

        // ===================================================================== //
        // Gaußscher Weichzeichner
        gaussianBlur(layer);

        // ===================================================================== //
        // Kanal als Auswahl laden
        loadChannelSelection(layer);

        // ===================================================================== //
        // Vorläufiges Selektives Schärfen
        preSeleciveSharp(layer);

        // ===================================================================== //
        // Finalisierung
        convertToRGBMode();
        recreateSettingsLayers();


    } catch (err) {
        alert(err);
        undoAction();
    }


    function getReferenceLayer() {

        if ($.layers.soft && $.layers.detail) {

            // ===================================================================== //
            // If there was a freq.sep we merge their layers and use the result as reference layer

            if ($.layers.levels) {
                $.layers.levels.visible = false;
            }

            $.layers.soft.visible = true;
            $.layers.detail.visible = true;

            // ===================================================================== //
            // When switching into LAB color mode, all settings layers will be deleted
            // So set this flag to recreate it using default values
            RecreateLevelsSettingsLayer = true;

            // ===================================================================== //
            return $.DocumentUtils.mergeLayers(app.activeDocument, [ $.layers.detail.duplicate(), $.layers.soft.duplicate() ]);

        } else {

            // ===================================================================== //
            // If there was no freq.sep. we use the background layer as reference layer
            return app.activeDocument.backgroundLayer.duplicate();

        }

    }

    function createLevelsLayer(rangeStart, rangeEnd) {

        // =======================================================
        var desc306 = new ActionDescriptor();
        var ref142 = new ActionReference();
        ref142.putClass(TypeIDs.AdjustLevel);
        desc306.putReference(TypeIDs.NULL, ref142);
        var idUsng = charIDToTypeID("Usng");
        var desc307 = new ActionDescriptor();
        var idType = charIDToTypeID("Type");
        var desc308 = new ActionDescriptor();
        desc308.putEnumerated(stringIDToTypeID("presetKind"), stringIDToTypeID("presetKindType"), stringIDToTypeID("presetKindDefault"));
        desc307.putObject(idType, TypeIDs.Levels, desc308);
        desc306.putObject(idUsng, TypeIDs.AdjustLevel, desc307);
        executeAction(TypeIDs.Make, desc306, DialogModes.NO);

        // =======================================================
        var desc251 = new ActionDescriptor();

        var ref112 = new ActionReference();
        ref112.putEnumerated(TypeIDs.AdjustLevel, charIDToTypeID("Ordn"), TypeIDs.Target);
        desc251.putReference(TypeIDs.NULL, ref112);

        var desc252 = new ActionDescriptor();
        desc252.putEnumerated(stringIDToTypeID("presetKind"), stringIDToTypeID("presetKindType"), stringIDToTypeID("presetKindCustom"));

        var list50 = new ActionList();
        var desc253 = new ActionDescriptor();
        var ref113 = new ActionReference();
        ref113.putEnumerated(TypeIDs.Channel, TypeIDs.Channel, charIDToTypeID("Cmps"));
        desc253.putReference(TypeIDs.Channel, ref113);

        var list51 = new ActionList();
        list51.putInteger(rangeStart);
        list51.putInteger(rangeEnd);
        desc253.putList(TypeIDs.Input, list51);

        list50.putObject(charIDToTypeID("LvlA"), desc253);
        desc252.putList(charIDToTypeID("Adjs"), list50);

        var idLvls = charIDToTypeID("Lvls");
        desc251.putObject(charIDToTypeID("T   "), idLvls, desc252);
        executeAction(TypeIDs.Set, desc251, DialogModes.NO);

        return app.activeDocument.activeLayer;
    }

    function convertLayerToSmartobject() {
        var idnewPlacedLayer = stringIDToTypeID("newPlacedLayer");
        executeAction(idnewPlacedLayer, undefined, DialogModes.NO);
    }

    function convertToLABMode() {
        var desc = new ActionDescriptor();
        desc.putClass(charIDToTypeID("T   "), charIDToTypeID("LbCM"));
        desc.putBoolean(charIDToTypeID("Fltt"), false);
        desc.putBoolean(charIDToTypeID("Rstr"), false);
        executeAction(charIDToTypeID("CnvM"), desc, DialogModes.NO);
    }

    function convertToRGBMode() {
        // =======================================================
        var desc = new ActionDescriptor();
        desc.putClass(charIDToTypeID("T   "), charIDToTypeID("RGBM"));
        desc.putBoolean(charIDToTypeID("Fltt"), false);
        desc.putBoolean(charIDToTypeID("Rstr"), false);
        executeAction(charIDToTypeID("CnvM"), desc, DialogModes.NO);
    }

    function duplicateBrightnessChannel() {
        var idDplc = charIDToTypeID("Dplc");
        var desc15 = new ActionDescriptor();
        var idnull = charIDToTypeID("null");
        var ref1 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idChnl = charIDToTypeID("Chnl");
        var idLght = charIDToTypeID("Lght");
        ref1.putEnumerated(idChnl, idChnl, idLght);
        desc15.putReference(idnull, ref1);
        executeAction(idDplc, desc15, DialogModes.NO);
    }

    function findContours(layer) {
        // ===================================================================== //
        // FInd contours on active layer
        var idFndE = charIDToTypeID("FndE");
        executeAction(idFndE, undefined, DialogModes.NO);
        // ===================================================================== //
        // Invert active layer
        var idInvr = charIDToTypeID("Invr");
        executeAction(idInvr, undefined, DialogModes.NO);
    }

    function levels() {
        // =======================================================
        var idLvls = charIDToTypeID("Lvls");
        var desc26 = new ActionDescriptor();
        var idpresetKind = stringIDToTypeID("presetKind");
        var idpresetKindType = stringIDToTypeID("presetKindType");
        var idpresetKindCustom = stringIDToTypeID("presetKindCustom");
        desc26.putEnumerated(idpresetKind, idpresetKindType, idpresetKindCustom);
        var idAdjs = charIDToTypeID("Adjs");
        var list1 = new ActionList();
        var desc27 = new ActionDescriptor();
        var idChnl = charIDToTypeID("Chnl");
        var ref7 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idOrdn = charIDToTypeID("Ordn");
        var idTrgt = charIDToTypeID("Trgt");
        ref7.putEnumerated(idChnl, idOrdn, idTrgt);
        desc27.putReference(idChnl, ref7);
        var idGmm = charIDToTypeID("Gmm ");
        desc27.putDouble(idGmm, 0.780000);
        var idLvlA = charIDToTypeID("LvlA");
        list1.putObject(idLvlA, desc27);
        desc26.putList(idAdjs, list1);
        executeAction(idLvls, desc26, DialogModes.ALL);
    }

    function gaussianBlur() {
        // =======================================================
        var idGsnB = charIDToTypeID("GsnB");
        var desc31 = new ActionDescriptor();
        var idRds = charIDToTypeID("Rds ");
        var idPxl = charIDToTypeID("#Pxl");
        desc31.putUnitDouble(idRds, idPxl, 1.000000);
        executeAction(idGsnB, desc31, DialogModes.ALL);
    }

    function loadChannelSelection() {
        // =======================================================
        var idsetd = charIDToTypeID("setd");
        var desc45 = new ActionDescriptor();
        var idnull = charIDToTypeID("null");
        var ref25 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idfsel = charIDToTypeID("fsel");
        ref25.putProperty(idChnl, idfsel);
        desc45.putReference(idnull, ref25);
        var idT = charIDToTypeID("T   ");
        var ref26 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idOrdn = charIDToTypeID("Ordn");
        var idTrgt = charIDToTypeID("Trgt");
        ref26.putEnumerated(idChnl, idOrdn, idTrgt);
        desc45.putReference(idT, ref26);
        executeAction(idsetd, desc45, DialogModes.NO);
        // =======================================================
        var idShw = charIDToTypeID("Shw ");
        var desc47 = new ActionDescriptor();
        var idnull = charIDToTypeID("null");
        var list2 = new ActionList();
        var ref29 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idChnl = charIDToTypeID("Chnl");
        var idLght = charIDToTypeID("Lght");
        ref29.putEnumerated(idChnl, idChnl, idLght);
        list2.putReference(ref29);
        var ref30 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idChnl = charIDToTypeID("Chnl");
        var idA = charIDToTypeID("A   ");
        ref30.putEnumerated(idChnl, idChnl, idA);
        list2.putReference(ref30);
        var ref31 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idChnl = charIDToTypeID("Chnl");
        var idB = charIDToTypeID("B   ");
        ref31.putEnumerated(idChnl, idChnl, idB);
        list2.putReference(ref31);
        desc47.putList(idnull, list2);
        executeAction(idShw, desc47, DialogModes.NO);
        // =======================================================
        var idHd = charIDToTypeID("Hd  ");
        var desc48 = new ActionDescriptor();
        var idnull = charIDToTypeID("null");
        var list3 = new ActionList();
        var ref32 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idOrdn = charIDToTypeID("Ordn");
        var idTrgt = charIDToTypeID("Trgt");
        ref32.putEnumerated(idChnl, idOrdn, idTrgt);
        list3.putReference(ref32);
        desc48.putList(idnull, list3);
        executeAction(idHd, desc48, DialogModes.NO);
        // =======================================================
        var idslct = charIDToTypeID("slct");
        var desc144 = new ActionDescriptor();
        var idnull = charIDToTypeID("null");
        var ref87 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idChnl = charIDToTypeID("Chnl");
        var idLab = charIDToTypeID("Lab ");
        ref87.putEnumerated(idChnl, idChnl, idLab);
        desc144.putReference(idnull, ref87);
        executeAction(idslct, desc144, DialogModes.NO);
    }

    function addToChannelSelection() {
        // =======================================================
        var idAdd = charIDToTypeID("Add ");
        var desc46 = new ActionDescriptor();
        var idnull = charIDToTypeID("null");
        var ref27 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idOrdn = charIDToTypeID("Ordn");
        var idTrgt = charIDToTypeID("Trgt");
        ref27.putEnumerated(idChnl, idOrdn, idTrgt);
        desc46.putReference(idnull, ref27);
        var idT = charIDToTypeID("T   ");
        var ref28 = new ActionReference();
        var idChnl = charIDToTypeID("Chnl");
        var idfsel = charIDToTypeID("fsel");
        ref28.putProperty(idChnl, idfsel);
        desc46.putReference(idT, ref28);
        executeAction(idAdd, desc46, DialogModes.NO);
    }

    function preSeleciveSharp() {
        // =======================================================
        var desc = new ActionDescriptor();
        desc.putUnitDouble(charIDToTypeID("Amnt"), charIDToTypeID("#Prc"), BEAUTYPANEL.config.unsharpMask.amount);
        desc.putUnitDouble(charIDToTypeID("Rds "), charIDToTypeID("#Pxl"), BEAUTYPANEL.config.unsharpMask.radius);
        desc.putInteger(charIDToTypeID("Thsh"), BEAUTYPANEL.config.unsharpMask.threshold);
        executeAction(charIDToTypeID("UnsM"), desc, DialogModes.ALL);
        // =======================================================
        var idsetd = charIDToTypeID("setd");
        var desc69 = new ActionDescriptor();
        var idnull = charIDToTypeID("null");
        var ref39 = new ActionReference();
        var idfilterFX = stringIDToTypeID("filterFX");
        ref39.putIndex(idfilterFX, 1);
        var idLyr = charIDToTypeID("Lyr ");
        var idOrdn = charIDToTypeID("Ordn");
        var idTrgt = charIDToTypeID("Trgt");
        ref39.putEnumerated(idLyr, idOrdn, idTrgt);
        desc69.putReference(idnull, ref39);
        var idfilterFX = stringIDToTypeID("filterFX");
        var desc70 = new ActionDescriptor();
        var idblendOptions = stringIDToTypeID("blendOptions");
        var desc71 = new ActionDescriptor();
        var idOpct = charIDToTypeID("Opct");
        var idPrc = charIDToTypeID("#Prc");
        desc71.putUnitDouble(idOpct, idPrc, 100.000000);
        var idMd = charIDToTypeID("Md  ");
        var idBlnM = charIDToTypeID("BlnM");
        var idLmns = charIDToTypeID("Lmns");
        desc71.putEnumerated(idMd, idBlnM, idLmns);
        var idblendOptions = stringIDToTypeID("blendOptions");
        desc70.putObject(idblendOptions, idblendOptions, desc71);
        var idfilterFX = stringIDToTypeID("filterFX");
        desc69.putObject(idfilterFX, idfilterFX, desc70);
        executeAction(idsetd, desc69, DialogModes.NO);
    }

    function recreateSettingsLayers() {
        if (RecreateLevelsSettingsLayer) {
            app.activeDocument.activeLayers = $.layers.detail;
            $.layers.levels = createLevelsLayer(112, 140);
            $.layers.levels.name = strings.layername_levels;
            $.layers.levels.visible = false;
        }
    }
}

maskedSharpen();
