this.SuS = this.SuS || {};
(function (exports) {
	'use strict';

	/**
	 * Collection of string command Ids for photoshop art tools
	 */
	var ToolName = {
	    marqueeSingleRowTool: "marqueeSingleRowTool",
	    marqueeSingleColumnTool: "marqueeSingleColumnTool",
	    lassoTool: "lassoTool",
	    polySelTool: "polySelTool",
	    magneticLassoTool: "magneticLassoTool",
	    quickSelectTool: "quickSelectTool",
	    magicWandTool: "magicWandTool",
	    cropTool: "cropTool",
	    sliceTool: "sliceTool",
	    sliceSelectTool: "sliceSelectTool",
	    spotHealingBrushTool: "spotHealingBrushTool",
	    magicStampTool: "magicStampTool",
	    patchSelection: "patchSelection",
	    redEyeTool: "redEyeTool",
	    paintbrushTool: "paintbrushTool",
	    pencilTool: "pencilTool",
	    colorReplacementBrushTool: "colorReplacementBrushTool",
	    cloneStampTool: "cloneStampTool",
	    patternStampTool: "patternStampTool",
	    historyBrushTool: "historyBrushTool",
	    artBrushTool: "artBrushTool",
	    eraserTool: "eraserTool",
	    backgroundEraserTool: "backgroundEraserTool",
	    magicEraserTool: "magicEraserTool",
	    gradientTool: "gradientTool",
	    bucketTool: "bucketTool",
	    blurTool: "blurTool",
	    sharpenTool: "sharpenTool",
	    smudgeTool: "smudgeTool",
	    dodgeTool: "dodgeTool",
	    burnInTool: "burnInTool",
	    saturationTool: "saturationTool",
	    penTool: "penTool",
	    freeformPenTool: "freeformPenTool",
	    addKnotTool: "addKnotTool",
	    deleteKnotTool: "deleteKnotTool",
	    convertKnotTool: "convertKnotTool",
	    typeCreateOrEditTool: "typeCreateOrEditTool",
	    typeVerticalCreateOrEditTool: "typeVerticalCreateOrEditTool",
	    typeCreateMaskTool: "typeCreateMaskTool",
	    typeVerticalCreateMaskTool: "typeVerticalCreateMaskTool",
	    pathComponentSelectTool: "pathComponentSelectTool",
	    directSelectTool: "directSelectTool",
	    rectangleTool: "rectangleTool",
	    roundedRectangleTool: "roundedRectangleTool",
	    ellipseTool: "ellipseTool",
	    polygonTool: "polygonTool",
	    lineTool: "lineTool",
	    customShapeTool: "customShapeTool",
	    textAnnotTool: "textAnnotTool",
	    soundAnnotTool: "soundAnnotTool",
	    eyedropperTool: "eyedropperTool",
	    colorSamplerToo: "colorSamplerToo",
	    rulerTool: "rulerTool",
	    handTool: "handTool",
	    zoomTool: "zoomTool"
	};

	/**
	 * Collection of CharIDs for photoshop commands
	 */
	var TypeIDs = {
	    NULL: charIDToTypeID("null"),
	    Target: charIDToTypeID("Trgt"),
	    Levels: charIDToTypeID('Lvls'),
	    Channel: charIDToTypeID("Chnl"),
	    Input: charIDToTypeID("Inpt"),

	    // Document
	    Layer: charIDToTypeID("Lyr "),

	    // Toolset
	    Brush: null,

	    // Units
	    ResultUnit: charIDToTypeID("#Rsl"),
	    Pixel: charIDToTypeID("#Pxl"),
	    Percent: charIDToTypeID("#Prc"),

	    // Values
	    Radius: charIDToTypeID("Rds "),
	    ResultValue: charIDToTypeID("Rslt"),

	    // Calculation
	    Add: charIDToTypeID('Add '),

	    // Actions
	    AdjustLevel: charIDToTypeID("AdjL"),
	    ApplyImage: charIDToTypeID('AppI'),
	    Make: charIDToTypeID('Mk  '),
	    Select: charIDToTypeID('slct'),
	    Set: charIDToTypeID("setd"),
	    CopyToLayer: charIDToTypeID("CpTL"),
	    Median: charIDToTypeID('Mdn '),
	    ImageSize: charIDToTypeID('ImgS'),
	    HighPass: charIDToTypeID('HghP'),
	    MakeVisible: charIDToTypeID('MkVs'),
	    Delete: charIDToTypeID('Dlt ')
	};

	// https://forums.adobe.com/thread/579195

	/**
	 * 
	 */
	function getTool () {
	    var ref = new ActionReference();
	    ref.putEnumerated(charIDToTypeID("capp"), charIDToTypeID("Ordn"), charIDToTypeID("Trgt"));
	    var cTool = typeIDToStringID(executeActionGet(ref).getEnumerationType(stringIDToTypeID('tool')));
	    return cTool;
	}

	// http://www.ps-scripts.com/bb/viewtopic.php?t=2179

	/**
	 * 
	 */
	function setTool(tool) {
	    var descriptor = new ActionDescriptor();
	    var reference = new ActionReference();

	    reference.putClass(app.stringIDToTypeID(tool));
	    descriptor.putReference(app.charIDToTypeID('null'), reference);

	    executeAction(app.charIDToTypeID('slct'), descriptor, DialogModes.NO);
	}

	/**
	 * 
	 * @param {string} name 
	 */
	function playAction(name) {
	    var idPly = charIDToTypeID("Ply ");
	    var desc1803 = new ActionDescriptor();
	    var idnull = charIDToTypeID("null");
	    var ref1178 = new ActionReference();
	    var idActn = charIDToTypeID("Actn");
	    ref1178.putName(idActn, name);
	    var idASet = charIDToTypeID("ASet");
	    ref1178.putName(idASet, "Standardaktionen");
	    desc1803.putReference(idnull, ref1178);
	    executeAction(idPly, desc1803, DialogModes.NO);
	}

	function currentToolName() {
		var ref = new ActionReference();
		ref.putProperty(stringIDToTypeID("property"), stringIDToTypeID("tool"));
		ref.putEnumerated(charIDToTypeID("capp"), charIDToTypeID("Ordn"), charIDToTypeID("Trgt"));
		var applicationDesc = executeActionGet(ref);
		//alert(typeIDToStringID(applicationDesc.getEnumerationType(stringIDToTypeID("tool"))));
		return (typeIDToStringID(applicationDesc.getEnumerationType(stringIDToTypeID("tool"))));
	}

	function findScript() {// Find the location where this script resides
		var where = "";
		try { FORCEERROR = FORCERRROR; }
		catch (err) { where = File(err.fileName); }
		return where;
	}

	function loadToolPresetSet(SetName) {
		var returncode = true;
		var scriptLocation = String(findScript());
		var path = scriptLocation.substr(0, scriptLocation.lastIndexOf("/") + 1);
	    var SetFile = new File(path + SetName + ".tpl");   // Requited to be in the script's folder
	    
		if (!SetFile.exists) { 
	        returncode = false; 
	    } else {
			var desc = new ActionDescriptor();
			var ref = new ActionReference();
			ref.putProperty(charIDToTypeID("Prpr"), stringIDToTypeID("toolPreset"));
			ref.putEnumerated(charIDToTypeID("capp"), charIDToTypeID("Ordn"), charIDToTypeID("Trgt"));
			desc.putReference(charIDToTypeID("null"), ref);
			desc.putPath(charIDToTypeID("T   "), new File(SetFile));
			desc.putBoolean(charIDToTypeID("Appe"), true);
			try { 
	            executeAction(charIDToTypeID("setd"), desc, DialogModes.NO); 
	        } catch (e) { 
	            returncode = false; 
	        }
	    }
	    
		return returncode;
	}

	function selectTool(tool, dialogMode) {
		var desc9 = new ActionDescriptor();
		var ref7 = new ActionReference();
		ref7.putClass(app.stringIDToTypeID(tool));
		desc9.putReference(app.charIDToTypeID('null'), ref7);
		executeAction(app.charIDToTypeID('slct'), desc9, dialogMode || DialogModes.NO);
	}

	/**
	 * 
	 * @param {string} PresetName 
	 */
	function selectToolPreset(PresetName) {
		var desc = new ActionDescriptor();
		var ref = new ActionReference();
		var idnull = charIDToTypeID("null");
		var idBrsh = charIDToTypeID("Brsh");
		ref.putName(idBrsh, PresetName);
		desc.putReference(idnull, ref);
		executeAction(charIDToTypeID("slct"), desc, DialogModes.NO);
	}

	function getToolRadius() {
	    // ToDo
	}

	function getCurrentBrushInfo() {
	    var brsh = {};

	    var ref = new ActionReference();
	    ref.putEnumerated(charIDToTypeID("capp"), charIDToTypeID("Ordn"), charIDToTypeID("Trgt"));
	    var currentBrush = executeActionGet(ref).getObjectValue(stringIDToTypeID("currentToolOptions")).getObjectValue(charIDToTypeID('Brsh'));

	    // Für alle abfangen für die es den parameter nicht gibt

	    brsh.diameter = currentBrush.getDouble(charIDToTypeID('Dmtr'));
	    brsh.hardness = currentBrush.getDouble(charIDToTypeID('Hrdn'));
	    brsh.angle = currentBrush.getDouble(charIDToTypeID('Angl'));
	    brsh.roundness = currentBrush.getDouble(charIDToTypeID('Rndn'));
	    brsh.spacing = currentBrush.getDouble(charIDToTypeID('Spcn'));

	    return brsh;
	}

	function setBrushInfo(brush) {

	    var idsetd = charIDToTypeID("setd");
	    var desc19 = new ActionDescriptor();
	    var idnull = charIDToTypeID("null");
	    var ref2 = new ActionReference();
	    var idBrsh = charIDToTypeID("Brsh");
	    var idOrdn = charIDToTypeID("Ordn");
	    var idTrgt = charIDToTypeID("Trgt");
	    ref2.putEnumerated(idBrsh, idOrdn, idTrgt);
	    desc19.putReference(idnull, ref2);
	    var idT = charIDToTypeID("T   ");
	    var desc = new ActionDescriptor();

	    var idPxl = charIDToTypeID("#Pxl");
	    var idPercent = charIDToTypeID("#Prc");

	    // set diameter
	    if (typeof brush.diameter === "number") {
	        var idmasterDiameter = charIDToTypeID('Dmtr');
	        desc.putUnitDouble(idmasterDiameter, idPxl, brush.diameter);
	    }

	    // set hardness
	    if (typeof brush.hardness === "number") {
	        var idHardness = charIDToTypeID("Hrdn");
	        desc.putUnitDouble(idHardness, idPercent, brush.hardness);
	    }

	    // Set Angle
	    if (typeof brush.angle === "number") {
	        var idRadius = charIDToTypeID("Angl");
	        desc.putDouble(idRadius, brush.radius);
	    }

	    // Set roundness
	    if (typeof brush.roundness === "number") {
	        var idRoundness = charIDToTypeID('Rndn');
	        desc.putUnitDouble(idRoundness, idPercent, brush.roundness);
	    }

	    // Set spacing
	    if (typeof brush.spacing === "number") {
	        var idSpacing = charIDToTypeID('Spcn');
	        desc.putDouble(idSpacing, brush.spacing);
	    }

	    // Set Flow
	    if (typeof brush.flow === "number") {
	        var idFlow = stringIDToTypeID("flow");
	        desc.putUnitDouble(idFlow, idPercent, brush.flow);
	    }

	    var idBrsh = charIDToTypeID("Brsh");
	    desc19.putObject(idT, idBrsh, desc);
	    executeAction(idsetd, desc19, DialogModes.ERROR);

	}

	function setCurrentToolMainOptions(options) {

	    if (!options) {
	        alert("setCurrentToolMainOptions: No options argument.");
	        return;
	    }

	    var desc = new ActionDescriptor();
	    var data = new ActionDescriptor();

	    // Set reference
	    var ref = new ActionReference();    
	    ref.putClass(stringIDToTypeID(app.currentTool));
	    desc.putReference(stringIDToTypeID("null"), ref);
	    
	    var idPercent = stringIDToTypeID("percentUnit");

	    // opacity
	    if (typeof options.opacity === "number") {
	        var idOpacity = stringIDToTypeID("opacity");
	        data.putUnitDouble(idOpacity, idPercent, options.opacity);
	    }

	    // flow  
	    if (typeof options.flow === "number") {
	        var idFlow = stringIDToTypeID("flow");
	        data.putUnitDouble(idFlow, idPercent, options.flow);
	    }

	    // Put data into descriptor
	    desc.putObject(stringIDToTypeID("to"), stringIDToTypeID("null"), data);

	    executeAction(stringIDToTypeID("set"), desc, DialogModes.ERROR);
	}

	/**
	 * 
	 * @param {string} layerName 
	 */
	function getLayerByName(layerName) {
	    var doc = app.activeDocument;

	    for (var i = 0; i < doc.layers.length; i++) {
	        var layer = doc.layers[i];
	        if (layer.name == layerName) {
	            return layer;
	        }
	    }

	    return null;
	}

	/**
	 * 
	 * @param {ArtLayer} sourceLayer 
	 */
	function copyLayer (sourceLayer) {
	    
	    if (typeof sourceLayer === "string") {
	        sourceLayer = getLayerByName(sourceLayer);
	    }
	    
	    for (var i = 1; i < arguments.length; i++) {
	  
	      var newLayer = getLayerByName(arguments[i]);
	  
	      if (newLayer) {
	          continue;
	      }
	  
	      newLayer = sourceLayer.duplicate();
	      newLayer.name = arguments[i];
	    }
	  
	  }

	/**
	 * 
	 * @param {ArtLayer|string} layerName 
	 */
	function deleteLayer (layerName) {
	    
	        var layer = getLayerByName(layerName);
	    
	        if (layer) {
	            layer.remove();
	        }
	    
	    }

	/**
	 * Merges all layers given as arguments.
	 * Selects first argument layer and adds all other to selection.
	 */
	function mergeLayers() {

	    if (arguments.length <= 0) {
	        alert("No layers to merge given");
	        return;
	    }

	    var name = arguments[0].name;

	    // Deselect all
	    app.activeDocument.activeLayer = arguments[0];

	    // Select layers to merge
	    for (var i = 0; i < arguments.length; i++) {

	        var layer = arguments[i];

	        var desc = new ActionDescriptor();
	        var ref = new ActionReference();
	        ref.putName(charIDToTypeID("Lyr "), layer.name);
	        desc.putReference(charIDToTypeID("null"), ref);
	        desc.putEnumerated(stringIDToTypeID("selectionModifier"), stringIDToTypeID("selectionModifierType"), stringIDToTypeID("addToSelection"));
	        desc.putBoolean(charIDToTypeID("MkVs"), false);
	      
	        app.executeAction(charIDToTypeID("slct"), desc, DialogModes.NO);

	    }

	    // Do the merge
	    app.executeAction(cTID("Mrg2"), new ActionDescriptor(), DialogModes.NO);

	    return SuS.Photoshop.getLayerByName(name);
	}

	/**
	 * 
	 * @param {*} layer 
	 * @param {*} radius 
	 * @param {*} threshold 
	 */
	function surfaceBlur(layer, radius, threshold, dialogMode) {

	    if (!app) {
	        return;
	    }

	    if (!dialogMode) {
	        dialogMode = DialogModes.ERROR;
	    }

	    var prevActiveLayer;
	    var desc;
	    var document = app.activeDocument;

	    if (!document) {
	        return;
	    }

	    // Overload surfaceBlur(layer : string, radius, threshold)
	    if (typeof layer === "string")
	        layer = getLayerByName(layer);

	    // Overload surfaceBlur(radius, threshold)
	    if (typeof layer === "number") {
	        threshold = radius;
	        radius = layer;
	    }

	    if (layer) {
	        prevActiveLayer = document.activeLayer;
	        document.activeLayer = layer;
	    }

	    desc = new ActionDescriptor();
	    desc.putUnitDouble(cTID('Rds '), cTID('#Pxl'), radius);
	    desc.putInteger(cTID('Thsh'), threshold);
	    executeAction(sTID('surfaceBlur'), desc, dialogMode);

	    if (layer) {
	        document.activeLayer = prevActiveLayer;
	    }

	}

	function findSnapshot(name) {
	    try {
	        var doc = app.activeDocument;
	        var historyStates = doc.historyStates;
	        var stateRef = historyStates.getByName(name);

	        if (stateRef) {
	            if (stateRef.snapshot) {
	                return stateRef;
	            }
	        }
	    } catch (ex) {

	    }

	    return null;
	}

	/**
	 * 
	 * @param {*} name 
	 */
	function createNamedSnapshot(name) {

	    var tempName = name;
	    var counter = 1;


	    while (findSnapshot(tempName)) {
	        tempName = name + " " + counter;
	        counter++;
	    }

	    name = tempName;

	    var desc = new ActionDescriptor();

	    var snapshotRef = new ActionReference();
	    snapshotRef.putClass(charIDToTypeID('SnpS'));
	    desc.putReference(charIDToTypeID('null'), snapshotRef);

	    var historySnapshotRef = new ActionReference();
	    historySnapshotRef.putProperty(charIDToTypeID('HstS'), charIDToTypeID('CrnH'));
	    desc.putReference(charIDToTypeID('From'), historySnapshotRef);

	    desc.putString(charIDToTypeID('Nm  '), name);
	    desc.putEnumerated(charIDToTypeID('Usng'), charIDToTypeID('HstS'), charIDToTypeID('FllD'));

	    executeAction(charIDToTypeID('Mk  '), desc, DialogModes.NO);

	    return name;
	}

	/**
	 * 
	 * @param {string} name 
	 */
	function removeNamedSnapshot(name) {

	    if (!findSnapshot(name)) {
	        return;
	    }

	    var desc = new ActionDescriptor();
	    var ref = new ActionReference();
	    ref.putName(charIDToTypeID('SnpS'), name);
	    desc.putReference(charIDToTypeID('null'), ref);
	    executeAction(charIDToTypeID('Dlt '), desc, DialogModes.NO);

	}

	/**
	 * 
	 * @param {string} name 
	 */
	function revertNamedSnapshot(name) {

	    if (!findSnapshot(name)) {
	        return;
	    }

	    var desc = new ActionDescriptor();
	    var ref = new ActionReference();
	    ref.putName(charIDToTypeID('SnpS'), name);
	    desc.putReference(charIDToTypeID('null'), ref);
	    executeAction(charIDToTypeID('slct'), desc, DialogModes.NO);
	    
	}

	function getHistoryStateByName(name) {

	    if (!name)
	        return;

	    return app.activeDocument.historyStates.getByName(name);
	}

	// Application

	exports.ToolName = ToolName;
	exports.TypeIDs = TypeIDs;
	exports.getTool = getTool;
	exports.setTool = setTool;
	exports.playAction = playAction;
	exports.currentToolName = currentToolName;
	exports.findScript = findScript;
	exports.loadToolPresetSet = loadToolPresetSet;
	exports.selectTool = selectTool;
	exports.selectToolPreset = selectToolPreset;
	exports.getToolRadius = getToolRadius;
	exports.getCurrentBrushInfo = getCurrentBrushInfo;
	exports.setBrushInfo = setBrushInfo;
	exports.setCurrentToolMainOptions = setCurrentToolMainOptions;
	exports.getLayerByName = getLayerByName;
	exports.copyLayer = copyLayer;
	exports.deleteLayer = deleteLayer;
	exports.mergeLayers = mergeLayers;
	exports.surfaceBlur = surfaceBlur;
	exports.createNamedSnapshot = createNamedSnapshot;
	exports.removeNamedSnapshot = removeNamedSnapshot;
	exports.revertNamedSnapshot = revertNamedSnapshot;
	exports.findSnapshot = findSnapshot;
	exports.getHistoryStateByName = getHistoryStateByName;

}((this.SuS.Photoshop = this.SuS.Photoshop || {})));
