/// <reference path="../functions.jsx" />
/// <reference path="../../res/strings.js" />

#include "../SuS.Photoshop.jsx"
#include "../prototype/Document.jsx"

function arrayContains(arr, item) {
    for (var i = 0; i < arr.length; i++) {
        if (arr[i] === item)
            return true;
    }
    return false;
}

function setEditPreset() {

    try {

        Beautypanel.refreshLayers();

        if (!("editPreset" in __config)) {
            return "No editPreset defined";
        }

        var SuS_PS = SuS.Photoshop;
        var document = app.activeDocument;
        var set = __config.editPreset;

        if ("visible-layers" in set) {

            var vLayers = set["visible-layers"];
            var missingLayers = [];

            // ===================================================================== //
            // Replace vLayer string keys with layer names
            for (var i = 0; i < vLayers.length; i++) {
                vLayers[i] = strings[vLayers[i]];
                var layer = document.getLayerByName(vLayers[i]);
                if (layer == null) {
                    missingLayers.push(vLayers[i]);
                }
            }

            // ===================================================================== //
            // Abort if there are one or more layers missing
            if (missingLayers.length > 0) {
                if ("missing-layer-msg" in set) {
                    alert(strings[set["missing-layer-msg"]].replace("$LAYER_NAME", missingLayers.join(', ')));
                }

                return "Missing layers " + missingLayers.join(', ');
            }

            // ===================================================================== //
            // Check for each layer if vLayers contains the layer and set its visibility'.
            for (var i = 0; i < document.layers.length; i++) {
                var layer = document.layers[i];
                layer.visible = arrayContains(vLayers, layer.name);
            }

        }

        if ("tool" in set) {
            app.currentTool = set.tool;
        }

        if ("opacity" in set) {
            SuS_PS.setCurrentToolMainOptions({
                opacity: set.opacity
            });
        }

        if ("hardness" in set) {
            // ===================================================================== //
            // Fetch current brushInfo 'cause setting hardness will only work when also setting diameter. Strange bug in Photoshop.
            var brushInfo = SuS_PS.getCurrentBrushInfo();

            SuS_PS.setBrushInfo({
                diameter: brushInfo.diameter,
                hardness: set.hardness
            });
        }

        if ("active-layer" in set) {
            var layer = document.getLayerByName(set["active-layer"]);
            if (layer) {
                document.activeLayer = layer;
            } else {
                return "Layer " + set["active-layer"] + " missing";
            }
        }

        return "";
    } catch (err) {
        return err;
    }

}

setEditPreset();
