/// <reference path="../functions.jsx" />

if (!$.LayerUtils) {

    /**
     * Functions extending Adobes ArtLayer. Not extending prototype.
     */
    $.LayerUtils = {

        applySaturation: function (layer) {
            return usingLayer(layer || this, function () {
                return app.executeAction(Codes.desaturate, undefined, DialogModes.ERROR);
            });
        },

        applyCalculation: function (layer, targetLayer, channel, invert, mode, scale, offset) {

            return usingLayer(layer || this, function () {

                // === Check parameters

                if (typeof channel === "string") {
                    channel = charIDToTypeID(channel);
                }
                if (typeof mode === "string") {
                    mode = charIDToTypeID(mode);
                }
                if (typeof invert !== "boolean") {
                    invert = false;
                }
                if (typeof scale !== "number") {
                    scale = 1;
                }
                if (typeof offset !== "number") {
                    offset = 0;
                }

                // ====================================================
                var actionDescriptpr = new ActionDescriptor();
                var parameterDescriptor = new ActionDescriptor();
                var reference = new ActionReference();
                reference.putEnumerated(Codes.channel, Codes.channel, channel);
                reference.putName(Codes.layer, targetLayer.name);
                parameterDescriptor.putReference(Codes.to, reference);
                parameterDescriptor.putBoolean(Codes.invert, true);
                parameterDescriptor.putEnumerated(Codes.calculation, Codes.calculation, mode);
                parameterDescriptor.putDouble(Codes.scale, 2.000000);
                parameterDescriptor.putInteger(Codes.offset, 0);
                actionDescriptpr.putObject(Codes.with, Codes.calculation, parameterDescriptor);

                return app.executeAction(Codes.applyImageEvent, actionDescriptpr, DialogModes.NO);

            });
        },

        createContrastLayer: function (layer, rangeStart, rangeEnd) {

            return usingLayer(layer, function () {

                // =======================================================
                // Create Adjustment Layer

                var desc306 = new ActionDescriptor();
                var ref142 = new ActionReference();
                ref142.putClass(Codes.adjustmentLayer);
                desc306.putReference(Codes.null, ref142);
                var desc307 = new ActionDescriptor();
                var desc308 = new ActionDescriptor();
                desc308.putEnumerated(Codes.presetKind, Codes.presetKindType, Codes.presetKindDefault);
                desc307.putObject(Codes.type, Codes.levels, desc308);
                desc306.putObject(Codes.using, Codes.adjustmentLayer, desc307);
                executeAction(Codes.make, desc306, DialogModes.ERROR);

                // =======================================================
                // Set Curves for Adjustment Layer

                var desc251 = new ActionDescriptor();

                var ref112 = new ActionReference();
                ref112.putEnumerated(Codes.adjustmentLayer, Codes.ordinal, Codes.target);
                desc251.putReference(Codes.null, ref112);

                var desc252 = new ActionDescriptor();
                desc252.putEnumerated(Codes.presetKind, Codes.presetKindType, Codes.presetKindCustom);

                var list50 = new ActionList();
                var desc253 = new ActionDescriptor();
                var ref113 = new ActionReference();
                ref113.putEnumerated(Codes.channel, Codes.channel, Codes.composite);
                desc253.putReference(Codes.channel, ref113);

                var list51 = new ActionList();
                list51.putInteger(rangeStart);
                list51.putInteger(rangeEnd);
                desc253.putList(Codes.input, list51);

                list50.putObject(Codes.levelsAdjustment, desc253);
                desc252.putList(Codes.adjustment, list50);

                desc251.putObject(Codes.to, Codes.levels, desc252);
                executeAction(Codes.set, desc251, DialogModes.ERROR);

                return layer.parent.activeLayer;

            })

        },

        createRvlaMask: function (layer) {

            return usingLayer(layer || this, function () {

                var reference = new ActionReference();
                reference.putEnumerated(Codes.channel, Codes.channel, Codes.mask);

                var desc = new ActionDescriptor();
                desc.putReference(Codes.at, reference);
                desc.putClass(Codes.new, Codes.channel);
                desc.putEnumerated(Codes.using, Codes.userMaskEnabled, Codes.revealAll);

                return app.executeAction(Codes.make, desc, DialogModes.ERROR);

            })

        },

        applySurfaceBlur: function (layer, radius, threshold, dialogMode) {
            return usingLayer(layer || this, function () {
                var descriptor = new ActionDescriptor();
                descriptor.putUnitDouble(Codes.radius, Codes.pixelsUnit, radius);
                descriptor.putInteger(Codes.threshold, threshold);
                return app.executeAction(stringIDToTypeID("surfaceBlur"), descriptor, dialogMode || app.displayDialogs);
            });
        }

    };

}