
if (typeof ($) == 'undefined')
    $ = {};

$._ext = {

    //Evaluate a file and catch the exception.
    evalFile: function (path) {
        try {
            $.evalFile(path);
        } catch (e) { alert("Exception:" + e); }
    },

    // Evaluate all the files in the given folder
    evalFiles: function (jsxFolderPath) {
        var folder = new Folder(jsxFolderPath);
        if (folder.exists) {
            var jsxFiles = folder.getFiles("*.jsx");
            for (var i = 0; i < jsxFiles.length; i++) {
                var jsxFile = jsxFiles[i];
                $._ext.evalFile(jsxFile);
            }
        }
    },

    readFile: function (url) {
        try {
            var f = new File(url);
            var content = "";

            f.open('r');
            while (!f.eof) {
                content += f.readln() + "\n";
            }
            f.close();

            return content;

        } catch (err) {
            return '{ "error": "' + err + '" }';
        }
    },
    writeFile: function (url, content) {

        try {

            var f = new File(url);

            if (f.exists)
                f.remove();

            f.encoding = "UTF8";

            f.open('w');
            if (f.error && f.error.length > 0)
                return f.error;

            f.write(content);
            if (f.error && f.error.length > 0)
                return f.error;

            f.close();

            if (f.error && f.error.length > 0)
                return f.error;

            return "";

        } catch (err) {
            return err;
        }

    },

};

// ======== CONSTANTS ==================

var STAGE_DEBUG = "d3bug";
var STAGE_RELEASE = "r3l3ase";

// ======== GLOBALS ====================

var STAGE = STAGE_DEBUG;

var BEAUTYPANEL = {

    files: {
        settings: Folder.userData.fullName + "/Stoehr+Sauer/Beautypanel/settings.wtf"
    },

    folders: {
        appData: Folder.userData.fullName + "/Stoehr+Sauer/Beautypanel"
    },

    config: {
        dialogMode: DialogModes.ERROR,
        unsharpMask: { amount: 100, radius: 2, threshold: 0 },
        vignette: {},
        save: { path: 0, scale: false, asPrevious: false },
        freqSep: { withDialogs: false }
    }

}

var strings = {};

function _setStrings(s) {
    if (s) {
        for (var key in s) {
            strings[key] = s[key];
        }
    }
}

function isRelease() {
    return STAGE.toLowerCase() === STAGE_RELEASE.toLowerCase();
}
function isDebug() {
    return STAGE.toLowerCase() === STAGE_DEBUG.toLowerCase();
}

// ======== Polyfills ==================

if (typeof JSON == "undefined") {
    JSON = {
        stringify: null,
        parse: null
    };
}

try {

    if (typeof JSON.parse !== "function") {
        JSON.parse = function (jsonCode) {
            return eval('(' + jsonCode + ')');
        }
    }

    if (typeof JSON.stringify !== "function") {
        JSON.stringify = (function () {
            var toString = Object.prototype.toString;
            var isArray = Array.isArray || function (a) { return toString.call(a) === '[object Array]'; };
            var escMap = { '"': '\\"', '\\': '\\\\', '\b': '\\b', '\f': '\\f', '\n': '\\n', '\r': '\\r', '\t': '\\t' };
            var escFunc = function (m) { return escMap[m] || '\\u' + (m.charCodeAt(0) + 0x10000).toString(16).substr(1); };
            var escRE = /[\\"\u0000-\u001F\u2028\u2029]/g;
            return function stringify(value) {
                if (value == null) {
                    return 'null';
                } else if (typeof value === 'number') {
                    return isFinite(value) ? value.toString() : 'null';
                } else if (typeof value === 'boolean') {
                    return value.toString();
                } else if (typeof value === 'object') {
                    if (typeof value.toJSON === 'function') {
                        return stringify(value.toJSON());
                    } else if (isArray(value)) {
                        var res = '[';
                        for (var i = 0; i < value.length; i++)
                            res += (i ? ', ' : '') + stringify(value[i]);
                        return res + ']';
                    } else if (toString.call(value) === '[object Object]') {
                        var tmp = [];
                        for (var k in value) {
                            if (value.hasOwnProperty(k))
                                tmp.push(stringify(k) + ': ' + stringify(value[k]));
                        }
                        return '{' + tmp.join(', ') + '}';
                    }
                }
                return '"' + value.toString().replace(escRE, escFunc) + '"';
            };
        })();
    }

} catch (err) {
    alert("Beautypanel: Error creating polyfills. " + err);
}

// --------- SuS Addins ----------------

/**
 *
 * @param {*} value
 */
function setDPI(value) {
    var descriptor = new ActionDescriptor();
    descriptor.putUnitDouble(Codes.resolution, Codes.densityUnit, value);
    executeAction(Codes.imageSize, descriptor, DialogModes.ERROR);
}

function cTID(s) {
    return app.charIDToTypeID(s);
}
function sTID(s) {
    return app.stringIDToTypeID(s);
}

function removeFileExtension(name) {
    if (!name.lastIndexOf) {
        alert("Error: String has no function called 'lastIndexOf'");
    }
    var lastDotIndex = name.lastIndexOf('.');
    if (lastDotIndex < 0) {
        return name;
    } else {
        return name.substring(0, lastDotIndex);
    }
}

// --------- CEP Action Codes ----------------

var Codes = {
    "128BitFloatingPoint": 'ldbl',
    "16BitsPerPixel": '16Bt',
    "1BitPerPixel": 'OnBt',
    "2BitsPerPixel": '2Bts',
    "32BitsPerPixel": '32Bt',
    "3DAntiAlias": 'Alis',
    "3DTransform": 'TdT ',
    "4BitsPerPixel": '4Bts',
    "5000": '5000',
    "5500": '5500',
    "6500": '6500',
    "72Color": '72Cl',
    "72Gray": '72Gr',
    "7500": '7500',
    "8BitsPerPixel": 'EghB',
    "9300": '9300',
    "ADSBottoms": 'AdBt',
    "ADSCentersH": 'AdCH',
    "ADSCentersV": 'AdCV',
    "ADSHorizontal": 'AdHr',
    "ADSLefts": 'AdLf',
    "ADSRights": 'AdRg',
    "ADSTops": 'AdTp',
    "ADSVertical": 'AdVr',
    "AEList": 'list',
    "AERecord": 'reco',
    "ASCII": 'ASCI',
    "CIERGB": 'CRGB',
    "CMYK": 'CMYK',
    "CMYK64": 'CMSF',
    "CMYKColorClass": 'CMYC',
    "CMYKColorEnum": 'ECMY',
    "CMYKColorMode": 'CMYM',
    "CMYKSetup": 'CMYS',
    "CMYKSetupEngine": 'CMYE',
    "DCS": 'DCS ',
    "Dicom": 'Dicm',
    "EPSGenericFormat": 'EPSG',
    "EPSPICTPreview": 'EPSC',
    "EPSPreview": 'EPSP',
    "EPSTIFFPreview": 'EPST',
    "FPXCompress": 'FxCm',
    "FPXCompressLossyJPEG": 'FxJP',
    "FPXCompressNone": 'FxNo',
    "FPXQuality": 'FxQl',
    "FPXSize": 'FxSz',
    "FPXView": 'FxVw',
    "FSS": 'fss ',
    "FuntaFormat": 'PSDX',
    "GCR": 'GCR ',
    "GIF89aExport": 'GF89',
    "GIFColorFileColorTable": 'GFCT',
    "GIFColorFileColors": 'GFCF',
    "GIFColorFileMicrosoftPalette": 'GFMS',
    "GIFColorFileType": 'GFPT',
    "GIFColorLimit": 'GFCL',
    "GIFExportCaption": 'GFEC',
    "GIFFormat": 'GFFr',
    "GIFMaskChannelIndex": 'GFMI',
    "GIFMaskChannelInverted": 'GFMV',
    "GIFPaletteAdaptive": 'GFPA',
    "GIFPaletteExact": 'GFPE',
    "GIFPaletteFile": 'GFPF',
    "GIFPaletteOther": 'GFPO',
    "GIFPaletteSystem": 'GFPS',
    "GIFPaletteType": 'GFPL',
    "GIFRequiredColorSpaceIndexed": 'GFCI',
    "GIFRequiredColorSpaceRGB": 'GFRG',
    "GIFRequiredColorSpaceType": 'GFCS',
    "GIFRowOrderInterlaced": 'GFIN',
    "GIFRowOrderNormal": 'GFNI',
    "GIFRowOrderType": 'GFIT',
    "GIFTransparentColor": 'GFTC',
    "GIFTransparentIndexBlue": 'GFTB',
    "GIFTransparentIndexGreen": 'GFTG',
    "GIFTransparentIndexRed": 'GFTR',
    "GIFUseBestMatch": 'GFBM',
    "HDTV": 'HDTV',
    "HSBColorClass": 'HSBC',
    "HSBColorEnum": 'HSBl',
    "HSBColorMode": 'HSBM',
    "HSLColor": 'HSLC',
    "IBMPC": 'IBMP',
    "ICC": 'ICC ',
    "ICCEngine": 'ICCE',
    "ICCSetupName": 'ICCt',
    "ID": 'Idnt',
    "IEEE32BitFloatingPoint": 'sing',
    "IEEE64BitFloatingPoint": 'doub',
    "JPEG": 'JPEG',
    "JPEGFormat": 'JPEG',
    "JPEGQuality": 'JPEQ',
    "JPS": 'JPS ',
    "LUTAnimation": 'LTnm',
    "LZWCompression": 'LZWC',
    "MPO": 'MPO ',
    "NTSC": 'NTSC',
    "NTSCColors": 'NTSC',
    "OS2": 'OS2 ',
    "P22EBU": 'P22B',
    "PDFGenericFormat": 'PDFG',
    "PICTFileFormat": 'PICF',
    "PICTResourceFormat": 'PICR',
    "PNGFilter": 'PNGf',
    "PNGFilterAdaptive": 'PGAd',
    "PNGFilterAverage": 'PGAv',
    "PNGFilterNone": 'PGNo',
    "PNGFilterPaeth": 'PGPt',
    "PNGFilterSub": 'PGSb',
    "PNGFilterUp": 'PGUp',
    "PNGFormat": 'PNGF',
    "PNGInterlaceAdam7": 'PGIA',
    "PNGInterlaceNone": 'PGIN',
    "PNGInterlaceType": 'PGIT',
    "QCSAverage": 'Qcsa',
    "QCSCorner0": 'Qcs0',
    "QCSCorner1": 'Qcs1',
    "QCSCorner2": 'Qcs2',
    "QCSCorner3": 'Qcs3',
    "QCSIndependent": 'Qcsi',
    "QCSSide0": 'Qcs4',
    "QCSSide1": 'Qcs5',
    "QCSSide2": 'Qcs6',
    "QCSSide3": 'Qcs7',
    "QDRectangle": 'qdrt',
    "RGB": 'RGB ',
    "RGB48": 'RGBF',
    "RGBColor": 'RGBC',
    "RGBColorMode": 'RGBM',
    "RGBSetup": 'RGBS',
    "RGBSetupClass": 'RGBt',
    "RGBSetupSource": 'RGBS',
    "SMPTEC": 'SMPC',
    "TIFF": 'TIFF',
    "TIFFFormat": 'TIFF',
    "UCA": 'UC  ',
    "URL": 'URL ',
    "a": 'A   ',
    "aboutApp": 'AbAp',
    "absColorimetric": 'AClr',
    "absolute": 'Absl',
    "accentedEdges": 'AccE',
    "action": 'Actn',
    "actionData": 'ActD',
    "actionReference": '#Act',
    "actionSet": 'ASet',
    "activeFrameIndex": 'AFrm',
    "activeFrameSetID": 'AFSt',
    "actualPixels": 'ActP',
    "adaptive": 'Adpt',
    "add": 'Add ',
    "addNoise": 'AdNs',
    "addTo": 'AddT',
    "addressAttr": 'addr',
    "adjustment": 'Adjs',
    "adjustmentLayer": 'AdjL',
    "adjustmentOptions": 'AdjO',
    "adobeRGB1998": 'SMPT',
    "airbrushEraser": 'Arbs',
    "airbrushTool": 'AbTl',
    "alias": 'alis',
    "align": 'Algn',
    "alignDistributeSelector": 'ADSt',
    "aligned": 'Algd',
    "alignment": 'Algn',
    "alignmentType": 'Alg ',
    "all": 'All ',
    "allEnum": 'Al  ',
    "allExcept": 'AllE',
    "allToolOptions": 'AlTl',
    "alphaChannelOptions": 'AChn',
    "alphaChannelOptionsClass": 'AChl',
    "alphaChannels": 'AlpC',
    "ambientBrightness": 'AmbB',
    "ambientColor": 'AmbC',
    "amount": 'Amnt',
    "amountHigh": 'amHi',
    "amountLow": 'amLo',
    "amountMedium": 'amMd',
    "amplitudeMax": 'AmMx',
    "amplitudeMin": 'AmMn',
    "anchor": 'Anch',
    "angle": 'Angl',
    "angle1": 'Ang1',
    "angle2": 'Ang2',
    "angle3": 'Ang3',
    "angle4": 'Ang4',
    "angleUnit": '#Ang',
    "angledStrokes": 'AngS',
    "animationFXRefPoint": 'FXRf',
    "animationFrameIDList": 'FrLs',
    "animationImageMask": 'IMsk',
    "animationLayerID": 'LaID',
    "animationLayerSettings": 'LaSt',
    "animationLayerSpecific": 'LSpc',
    "animationVectorMask": 'VMsk',
    "antiAlias": 'AntA',
    "antiAliasCrisp": 'AnCr',
    "antiAliasHigh": 'AnHi',
    "antiAliasLow": 'AnLo',
    "antiAliasMedium": 'AnMd',
    "antiAliasNone": 'Anno',
    "antiAliasSmooth": 'AnSm',
    "antiAliasStrong": 'AnSt',
    "antiAliasType": 'Annt',
    "antiAliasedPICTAcquire": 'AntA',
    "any": 'Any ',
    "appParameters": 'appa',
    "append": 'Appe',
    "applSignature": 'sign',
    "appleEvent": 'aevt',
    "appleRGB": 'AppR',
    "application": 'capp',
    "apply": 'Aply',
    "applyImageEnum": 'AplI',
    "applyImageEvent": 'AppI',
    "applyStyle": 'ASty',
    "area": 'Ar  ',
    "areaSelector": 'ArSl',
    "aroundCenter": 'ArnC',
    "arrange": 'Arng',
    "arrowhead": 'cArw',
    "artBrushTool": 'ABTl',
    "as": 'As  ',
    "ask": 'Ask ',
    "askWhenOpening": 'AskW',
    "assert": 'Asrt',
    "assumeOptions": 'AssO',
    "assumedCMYK": 'AssC',
    "assumedGray": 'AssG',
    "assumedProfile": 'AssP',
    "assumedRGB": 'AssR',
    "at": 'At  ',
    "authorName": 'Athr',
    "auto": 'Auto',
    "autoContrast": 'AuCo',
    "autoErase": 'Atrs',
    "autoKern": 'AtKr',
    "autoUpdateFiles": 'AtUp',
    "axis": 'Axis',
    "b": 'B   ',
    "bMPFormat": 'BMPF',
    "back": 'Back',
    "background": 'Bckg',
    "backgroundColor": 'BckC',
    "backgroundEraserTool": 'SETl',
    "backgroundLayer": 'BckL',
    "backgroundLevel": 'BckL',
    "backward": 'Bwd ',
    "backwardEnum": 'Bckw',
    "balance": 'Blnc',
    "basRelief": 'BsRl',
    "baselineShift": 'Bsln',
    "batch": 'Btch',
    "batchFromDroplet": 'BtcF',
    "beepWhenDone": 'BpWh',
    "beforeRunning": 'hbr ',
    "beginRamp": 'BgnR',
    "beginSustain": 'BgnS',
    "behind": 'Bhnd',
    "best": 'Bst ',
    "better": 'Dthb',
    "bevelDirection": 'bvlD',
    "bevelEmboss": 'ebbl',
    "bevelEmbossStampStyle": 'BESs',
    "bevelEmbossStyle": 'BESl',
    "bevelStyle": 'bvlS',
    "bevelTechnique": 'bvlT',
    "bicubic": 'Bcbc',
    "bigNudgeH": 'BgNH',
    "bigNudgeV": 'BgNV',
    "bilinear": 'Blnr',
    "binary": 'Bnry',
    "bitDepth": 'BtDp',
    "bitDepth1": 'BD1 ',
    "bitDepth24": 'BD24',
    "bitDepth4": 'BD4 ',
    "bitDepth8": 'BD8 ',
    "bitmap": 'Btmp',
    "bitmapMode": 'BtmM',
    "black": 'Blck',
    "blackAndWhite": 'BanW',
    "blackBody": 'BlcB',
    "blackClip": 'BlcC',
    "blackGeneration": 'Blcn',
    "blackGenerationCurve": 'BlcG',
    "blackGenerationType": 'BlcG',
    "blackIntensity": 'BlcI',
    "blackLevel": 'BlcL',
    "blackLimit": 'BlcL',
    "blacks": 'Blks',
    "blast": 'Blst',
    "bleed": 'Bld ',
    "blendMode": 'BlnM',
    "blendRange": 'Blnd',
    "blockEraser": 'Blk ',
    "blocks": 'Blks',
    "blue": 'Bl  ',
    "blueBlackPoint": 'BlBl',
    "blueGamma": 'BlGm',
    "blueWhitePoint": 'BlWh',
    "blueX": 'BlX ',
    "blueY": 'BlY ',
    "blues": 'Bls ',
    "blur": 'blur',
    "blurEvent": 'Blr ',
    "blurMethod": 'BlrM',
    "blurMore": 'BlrM',
    "blurQuality": 'BlrQ',
    "blurTool": 'BlTl',
    "book": 'Bk  ',
    "bookColor": 'BkCl',
    "boolean": 'bool',
    "border": 'Brdr',
    "borderThickness": 'BrdT',
    "bottom": 'Btom',
    "bottomEnum": 'Bttm',
    "brightness": 'Brgh',
    "brightnessContrast": 'BrgC',
    "brightnessEvent": 'BrgC',
    "brush": 'Brsh',
    "brushDarkRough": 'BrDR',
    "brushDetail": 'BrsD',
    "brushLightRough": 'BrsL',
    "brushSimple": 'BrSm',
    "brushSize": 'BrsS',
    "brushSparkle": 'BrSp',
    "brushType": 'BrsT',
    "brushWideBlurry": 'BrbW',
    "brushWideSharp": 'BrsW',
    "brushes": 'Brsh',
    "brushesAppend": 'BrsA',
    "brushesDefine": 'BrsD',
    "brushesDelete": 'Brsf',
    "brushesLoad": 'Brsd',
    "brushesNew": 'BrsN',
    "brushesOptions": 'BrsO',
    "brushesReset": 'BrsR',
    "brushesSave": 'Brsv',
    "builtInShapeCurve": 'BltC',
    "builtin": 'Bltn',
    "builtinProfile": 'BltP',
    "bumpAmplitude": 'BmpA',
    "bumpChannel": 'BmpC',
    "burnInH": 'BrnH',
    "burnInM": 'BrnM',
    "burnInS": 'BrnS',
    "burnInTool": 'BrTl',
    "buttonMode": 'BtnM',
    "by": 'By  ',
    "byline": 'Byln',
    "bylineTitle": 'BylT',
    "byteOrder": 'BytO',
    "cachePrefs": 'CchP',
    "calculation": 'Clcl',
    "calculationType": 'Clcn',
    "calculations": 'Clcl',
    "calibrationBars": 'Clbr',
    "canvasSize": 'CnvS',
    "caption": 'Cptn',
    "captionWriter": 'CptW',
    "cascade": 'Cscd',
    "category": 'Ctgr',
    "cellSize": 'ClSz',
    "center": 'Cntr',
    "centerCropMarks": 'CntC',
    "centerGlow": 'SrcC',
    "centeredFrame": 'CtrF',
    "chalkArea": 'ChlA',
    "chalkCharcoal": 'ChlC',
    "channel": 'Chnl',
    "channelMatrix": 'ChMx',
    "channelMixer": 'ChnM',
    "channelName": 'ChnN',
    "channelOptions": 'ChnO',
    "channelReference": '#ChR',
    "channels": 'Chns',
    "channelsInterleaved": 'ChnI',
    "channelsPaletteOptions": 'ChnP',
    "char": 'TEXT',
    "charcoal": 'Chrc',
    "charcoalAmount": 'ChAm',
    "charcoalArea": 'ChrA',
    "checkerboardLarge": 'ChcL',
    "checkerboardMedium": 'ChcM',
    "checkerboardNone": 'ChcN',
    "checkerboardSize": 'Chck',
    "checkerboardSmall": 'ChcS',
    "chokeMatte": 'Ckmt',
    "chrome": 'Chrm',
    "chromeFX": 'ChFX',
    "city": 'City',
    "class": 'type',
    "classColor": '#Clr',
    "classElement": '#ClE',
    "classExport": '#Cle',
    "classFormat": '#ClF',
    "classHueSatHueSatV2": '#HsV',
    "classImport": '#ClI',
    "classMode": '#ClM',
    "classStringFormat": '#ClS',
    "classTextExport": '#CTE',
    "classTextImport": '#ClT',
    "clearAmount": 'ClrA',
    "clearEnum": 'Clar',
    "clearEvent": 'Cler',
    "clearGuides": 'ClrG',
    "clipboard": 'Clpb',
    "clippingInfo": 'Clpo',
    "clippingPath": 'ClPt',
    "clippingPathEPS": 'ClpP',
    "clippingPathFlatness": 'ClpF',
    "clippingPathIndex": 'ClpI',
    "clippingPathInfo": 'Clpg',
    "cloneSource": 'ClnS',
    "cloneStampTool": 'ClTl',
    "close": 'Cls ',
    "closeAll": 'ClsA',
    "closedSubpath": 'Clsp',
    "clouds": 'Clds',
    "coarseDots": 'CrsD',
    "color": 'Clr ',
    "colorBalance": 'ClrB',
    "colorBurn": 'CBrn',
    "colorChannel": '#ClC',
    "colorChannels": 'Clrh',
    "colorCorrection": 'ClrC',
    "colorDodge": 'CDdg',
    "colorHalftone": 'ClrH',
    "colorIndicates": 'ClrI',
    "colorManagement": 'ClMg',
    "colorMatch": 'ClMt',
    "colorNoise": 'ClNs',
    "colorPalette": 'ClrP',
    "colorPickerPrefs": 'Clrr',
    "colorPickerPrefsClass": 'Clrk',
    "colorRange": 'ClrR',
    "colorSampler": 'ClSm',
    "colorSpace": 'ClrS',
    "colorStop": 'Clrt',
    "colorStopType": 'Clry',
    "colorTable": 'ClrT',
    "coloredPencil": 'ClrP',
    "colorimetric": 'Clrm',
    "colorize": 'Clrz',
    "colors": 'Clrs',
    "colorsList": 'ClrL',
    "columnWidth": 'ClmW',
    "command": 'Cmnd',
    "commandKey": 'CmdK',
    "comp": 'comp',
    "compensation": 'Cmpn',
    "composite": 'Cmps',
    "compression": 'Cmpr',
    "concavity": 'Cncv',
    "condition": 'Cndt',
    "constant": 'Cnst',
    "constrain": 'Cnst',
    "constrainProportions": 'CnsP',
    "constructionFOV": 'Cfov',
    "conteCrayon": 'CntC',
    "contiguous": 'Cntg',
    "continue": 'Cntn',
    "continuity": 'Cnty',
    "contourEdge": 'CntE',
    "contract": 'Cntc',
    "contrast": 'Cntr',
    "convert": 'Cnvr',
    "convertMode": 'CnvM',
    "convertToCMYK": 'CnvC',
    "convertToGray": 'CnvG',
    "convertToLab": 'CnvL',
    "convertToRGB": 'CnvR',
    "copy": 'Cpy ',
    "copyEffects": 'CpFX',
    "copyEvent": 'copy',
    "copyMerged": 'CpyM',
    "copyToLayer": 'CpTL',
    "copyright": 'Cpyr',
    "copyrightNotice": 'CprN',
    "cornerCropMarks": 'CrnC',
    "cornerRadiusUnit": 'RadU',
    "correctionMethod": 'CrcM',
    "count": 'Cnt ',
    "countryName": 'CntN',
    "crackBrightness": 'CrcB',
    "crackDepth": 'CrcD',
    "crackSpacing": 'CrcS',
    "craquelure": 'Crql',
    "createDroplet": 'CrtD',
    "createDuplicate": 'CrtD',
    "createInterpolation": 'CrtI',
    "createLayersFromLayerFX": 'blfl',
    "creatorAddr": 'Crad',
    "creatorAddrCity": 'Crci',
    "creatorAddrCountry": 'Crco',
    "creatorAddrPostCode": 'Crpc',
    "creatorAddrRegion": 'Crar',
    "creatorEmail": 'Crem',
    "creatorPhone": 'Crph',
    "creatorURL": 'Crur',
    "credit": 'Crdt',
    "crop": 'Crop',
    "cross": 'Crs ',
    "crosshatch": 'Crsh',
    "crossover": 'Crss',
    "crystallize": 'Crst',
    "current": 'Crnt',
    "currentHistoryState": 'CrnH',
    "currentLayer": 'CrrL',
    "currentLight": 'CrnL',
    "currentToolOptions": 'CrnT',
    "cursorKind": 'CrsK',
    "curve": 'Crv ',
    "curveFile": 'CrvF',
    "curvePoint": 'CrPt',
    "curves": 'Crvs',
    "curvesAdjustment": 'CrvA',
    "custom": 'Cstm',
    "customEnum": 'Cst ',
    "customForced": 'CstF',
    "customMatte": 'CstM',
    "customPalette": 'CstP',
    "customPaletteClass": 'Cstl',
    "customPattern": 'Cstm',
    "customPhosphors": 'CstP',
    "customStops": 'CstS',
    "customWhitePoint": 'CstW',
    "cut": 'cut ',
    "cutToLayer": 'CtTL',
    "cutout": 'Ct  ',
    "cyan": 'Cyn ',
    "cyans": 'Cyns',
    "dark": 'Drk ',
    "darkIntensity": 'DrkI',
    "darkStrokes": 'DrkS',
    "darken": 'Drkn',
    "darkenOnly": 'DrkO',
    "darkness": 'Drkn',
    "dashedLines": 'DshL',
    "dateCreated": 'DtCr',
    "datum": 'Dt  ',
    "deInterlace": 'Dntr',
    "debugExecutionMode": 'XMde',
    "debugExecutionModeClass": 'dbex',
    "decimalStruct": 'decm',
    "deepDepth": 'DpDp',
    "definePattern": 'DfnP',
    "definition": 'Dfnt',
    "defringe": 'Dfrg',
    "delete": 'Dlt ',
    "density": 'Dnst',
    "densityUnit": '#Rsl',
    "depth": 'Dpth',
    "desaturate": 'Dstt',
    "deselect": 'Dslc',
    "despeckle": 'Dspc',
    "destBlackMax": 'Dstl',
    "destBlackMin": 'DstB',
    "destWhiteMax": 'Dstt',
    "destWhiteMin": 'DstW',
    "destinationMode": 'DstM',
    "detail": 'Dtl ',
    "diameter": 'Dmtr',
    "diamond": 'Dmnd',
    "difference": 'Dfrn',
    "differenceClouds": 'DfrC',
    "diffuse": 'Dfs ',
    "diffuseGlow": 'DfsG',
    "diffuseMode": 'DfsM',
    "diffusion": 'Dfsn',
    "diffusionDither": 'DffD',
    "diffusionDitherEnum": 'DfnD',
    "direction": 'Drct',
    "directionBalance": 'DrcB',
    "disableLayerFX": 'dlfx',
    "disableSingleFX": 'dsfx',
    "dispatcherID": 'dspt',
    "displace": 'Dspl',
    "displaceFile": 'DspF',
    "displacementMap": 'DspM',
    "displayCursorsPreferences": 'DspC',
    "displayPrefs": 'DspP',
    "disposeFrame": 'Disp',
    "dissolve": 'Dslv',
    "distance": 'Dstn',
    "distanceUnit": '#Rlt',
    "distort": 'Dstr',
    "distortion": 'Dstr',
    "distribute": 'Dstr',
    "distribution": 'Dstr',
    "dither": 'Dthr',
    "ditherAmount": 'DthA',
    "ditherPreserve": 'Dthp',
    "ditherQuality": 'Dthq',
    "dlDocInfoChanged": 'dlDI',
    "document": 'Dcmn',
    "documentID": 'DocI',
    "documentReference": '#DcR',
    "dodgeH": 'DdgH',
    "dodgeM": 'DdgM',
    "dodgeS": 'DdgS',
    "dodgeTool": 'DdTl',
    "dotGain": 'DtGn',
    "dotGainCurves": 'DtGC',
    "dots": 'Dts ',
    "draft": 'Drft',
    "draw": 'Draw',
    "dropShadow": 'DrSh',
    "dryBrush": 'DryB',
    "duotone": 'Dtn ',
    "duotoneInk": 'DtnI',
    "duotoneMode": 'DtnM',
    "duplicate": 'Dplc',
    "dustAndScratches": 'DstS',
    "dynamicColorSliders": 'DnmC',
    "eBUITU": 'EBT ',
    "edge": 'Edg ',
    "edgeBrightness": 'EdgB',
    "edgeFidelity": 'EdgF',
    "edgeGlow": 'SrcE',
    "edgeIntensity": 'EdgI',
    "edgeSimplicity": 'EdgS',
    "edgeThickness": 'EdgT',
    "edgeWidth": 'EdgW',
    "effect": 'Effc',
    "element": 'Elmn',
    "elementReference": '#ElR',
    "eliminateEvenFields": 'ElmE',
    "eliminateOddFields": 'ElmO',
    "ellipse": 'Elps',
    "embedCMYK": 'EmbC',
    "embedGray": 'EmbG',
    "embedLab": 'EmbL',
    "embedProfiles": 'EmbP',
    "embedRGB": 'EmbR',
    "emboss": 'Embs',
    "emulsionDown": 'EmlD',
    "enableGestures": 'EGst',
    "enableLargeTabs": 'ELgt',
    "enabled": 'enab',
    "encoding": 'Encd',
    "end": 'End ',
    "endArrowhead": 'EndA',
    "endRamp": 'EndR',
    "endSustain": 'EndS',
    "engine": 'Engn',
    "enumerated": 'enum',
    "equalize": 'Eqlz',
    "eraseToHistory": 'ErsT',
    "eraserKind": 'ErsK',
    "eraserTool": 'ErTl',
    "eventClassAttr": 'evcl',
    "eventIDAttr": 'evid',
    "eventRecord": 'evrc',
    "eventSourceAttr": 'esrc',
    "exact": 'Exct',
    "exactPoints": 'ExcP',
    "exchange": 'Exch',
    "exclusion": 'Xclu',
    "expand": 'Expn',
    "export": 'Expr',
    "exportClipboard": 'ExpC',
    "exposure": 'Exps',
    "extend": 'Extd',
    "extended": 'exte',
    "extendedQuality": 'EQlt',
    "extension": 'Extn',
    "extensionsQuery": 'ExtQ',
    "extrude": 'Extr',
    "extrudeDepth": 'ExtD',
    "extrudeMaskIncomplete": 'ExtM',
    "extrudeRandom": 'ExtR',
    "extrudeSize": 'ExtS',
    "extrudeSolidFace": 'ExtF',
    "extrudeType": 'ExtT',
    "eyeDropperSample": 'EyDr',
    "eyeDropperSampleType": 'EyDp',
    "facet": 'Fct ',
    "fade": 'Fade',
    "fadeTo": 'FdT ',
    "fadeoutSteps": 'FdtS',
    "falloff": 'FlOf',
    "false": 'fals',
    "faster": 'Dthf',
    "feather": 'Fthr',
    "fiberLength": 'FbrL',
    "file": 'File',
    "fileCreator": 'FlCr',
    "fileEnum": 'Fle ',
    "fileInfo": 'FlIn',
    "fileReference": 'FilR',
    "fileSavePrefs": 'FlSP',
    "fileSavePrefsClass": 'FlSv',
    "fileType": 'FlTy',
    "filesList": 'flst',
    "fill": 'Fl  ',
    "fillBack": 'FlBc',
    "fillColor": 'FlCl',
    "fillContents": 'FlCn',
    "fillFore": 'FlFr',
    "fillInverse": 'FlIn',
    "fillMode": 'FlMd',
    "fillNeutral": 'FlNt',
    "fillSame": 'FlSm',
    "filmGrain": 'FlmG',
    "filter": 'Fltr',
    "findEdges": 'FndE',
    "fineDots": 'FnDt',
    "fingerpainting": 'Fngr',
    "first": 'Frst',
    "firstIdle": 'FrId',
    "fitOnScreen": 'FtOn',
    "fixed": 'fixd',
    "flareCenter": 'FlrC',
    "flashPixFormat": 'FlsP',
    "flatness": 'Fltn',
    "flatten": 'Fltt',
    "flattenImage": 'FltI',
    "flip": 'Flip',
    "floatType": 'doub',
    "floatUnit": 'UntF',
    "focus": 'Fcs ',
    "folders": 'Fldr',
    "fontDesignAxes": 'FntD',
    "fontDesignAxesVectors": 'FntV',
    "fontName": 'FntN',
    "fontScript": 'Scrp',
    "fontStyleName": 'FntS',
    "fontTechnology": 'FntT',
    "forcedColors": 'FrcC',
    "forcedNoDebugger": 'hfnd',
    "foregroundColor": 'FrgC',
    "foregroundLevel": 'FrgL',
    "format": 'Fmt ',
    "forward": 'Fwd ',
    "forwardEnum": 'Frwr',
    "fragment": 'Frgm',
    "frameDelay": 'FrDl',
    "frameDispose": 'FrDs',
    "frameFX": 'FrFX',
    "frameFill": 'FrFl',
    "frameGlobalAltitude": 'FrGL',
    "frameGlobalAngle": 'FrGA',
    "frameID": 'FrID',
    "frameInfo": 'FrIn',
    "frameSetID": 'FsID',
    "frameSets": 'FSts',
    "frameStyle": 'FStl',
    "frameWidth": 'FrmW',
    "frames": 'FsFr',
    "freeTransform": 'FrTr',
    "freeTransformCenterState": 'FTcs',
    "frequency": 'Frqn',
    "fresco": 'Frsc',
    "from": 'From',
    "fromBuiltin": 'FrmB',
    "fromMode": 'FrmM',
    "front": 'Frnt',
    "fullDocument": 'FllD',
    "fullSize": 'FlSz',
    "functionKey": 'FncK',
    "fuzziness": 'Fzns',
    "gamma": 'Gmm ',
    "gamutWarning": 'GmtW',
    "gaussianBlur": 'GsnB',
    "gaussianDistribution": 'Gsn ',
    "generalPreferences": 'GnrP',
    "generalPrefs": 'GnrP',
    "get": 'getd',
    "glass": 'Gls ',
    "globalAngle": 'gblA',
    "globalClass": 'GlbC',
    "globalLightingAngle": 'gagl',
    "globalObject": 'GlbO',
    "gloss": 'Glos',
    "glowAmount": 'GlwA',
    "glowTechnique": 'GlwT',
    "glowingEdges": 'GlwE',
    "good": 'Gd  ',
    "gradient": 'Grad',
    "gradientClassEvent": 'Grdn',
    "gradientFill": 'GrFl',
    "gradientForm": 'GrdF',
    "gradientMapClass": 'GdMp',
    "gradientMapEvent": 'GrMp',
    "gradientTool": 'GrTl',
    "gradientType": 'GrdT',
    "grain": 'Grn ',
    "grainClumped": 'GrnC',
    "grainContrasty": 'GrCn',
    "grainEnlarged": 'GrnE',
    "grainHorizontal": 'GrnH',
    "grainRegular": 'GrnR',
    "grainSoft": 'GrSf',
    "grainSpeckle": 'GrSp',
    "grainSprinkles": 'GrSr',
    "grainStippled": 'GrSt',
    "grainType": 'Grnt',
    "grainVertical": 'GrnV',
    "graininess": 'Grns',
    "grainyDots": 'GrnD',
    "graphicPen": 'GraP',
    "graphics": 'Grp ',
    "gray": 'Gry ',
    "gray16": 'GryX',
    "gray18": 'Gr18',
    "gray22": 'Gr22',
    "gray50": 'Gr50',
    "grayBehavior": 'GrBh',
    "grayScale": 'Gryc',
    "graySetup": 'GrSt',
    "grayscale": 'Grsc',
    "grayscaleMode": 'Grys',
    "green": 'Grn ',
    "greenBlackPoint": 'GrnB',
    "greenGamma": 'GrnG',
    "greenWhitePoint": 'GrnW',
    "greenX": 'GrnX',
    "greenY": 'GrnY',
    "greens": 'Grns',
    "gridColor": 'GrdC',
    "gridCustomColor": 'Grds',
    "gridMajor": 'GrdM',
    "gridMinor": 'Grdn',
    "gridStyle": 'GrdS',
    "gridUnits": 'Grdt',
    "group": 'Grup',
    "groupEvent": 'GrpL',
    "groutWidth": 'GrtW',
    "grow": 'Grow',
    "guide": 'Gd  ',
    "guideGridColor": 'GdGr',
    "guideGridStyle": 'GdGS',
    "guides": 'Gdes',
    "guidesColor": 'GdsC',
    "guidesCustomColor": 'Gdss',
    "guidesGridPreferences": 'GudG',
    "guidesPrefs": 'GdPr',
    "guidesStyle": 'GdsS',
    "gutterWidth": 'GttW',
    "halftoneFile": 'HlfF',
    "halftoneScreen": 'HlfS',
    "halftoneSize": 'HlSz',
    "halftoneSpec": 'Hlfp',
    "hardLight": 'HrdL',
    "hardness": 'Hrdn',
    "header": 'Hdr ',
    "headline": 'Hdln',
    "heavy": 'Hvy ',
    "height": 'Hght',
    "heightUnit": 'HgtU',
    "hide": 'Hd  ',
    "hideAll": 'HdAl',
    "hideSelection": 'HdSl',
    "high": 'High',
    "highPass": 'HghP',
    "highQuality": 'Hgh ',
    "highlightArea": 'HghA',
    "highlightColor": 'hglC',
    "highlightColorOption": 'UIHi',
    "highlightColorOptionEnumType": 'UIHe',
    "highlightLevels": 'HghL',
    "highlightMode": 'hglM',
    "highlightOpacity": 'hglO',
    "highlightStrength": 'HghS',
    "highlights": 'Hghl',
    "histogram": 'Hstg',
    "history": 'Hsty',
    "historyBrushSource": 'HstB',
    "historyBrushTool": 'HBTl',
    "historyPaletteOptions": 'HstO',
    "historyPreferences": 'HstP',
    "historyPrefs": 'HstP',
    "historyPrefsClass": 'CHsP',
    "historyState": 'HstS',
    "historyStateSource": 'HsSS',
    "historyStateSourceType": 'HstS',
    "historyStates": 'HsSt',
    "horizontal": 'Hrzn',
    "horizontalLocation": 'HrzL',
    "horizontalOnly": 'HrzO',
    "horizontalScale": 'HrzS',
    "hostName": 'HstN',
    "hostVersion": 'HstV',
    "hue": 'H   ',
    "hueSatAdjustment": 'HStA',
    "hueSatAdjustmentV2": 'Hst2',
    "hueSaturation": 'HStr',
    "icon": 'Icn ',
    "idle": 'Idle',
    "idleVM": 'IdVM',
    "ignore": 'Ignr',
    "illustratorPathsExport": 'IlsP',
    "image": 'Img ',
    "imageBalance": 'ImgB',
    "imageCachePreferences": 'ImgP',
    "imageLocation": 'ImLo',
    "imagePoint": 'ImgP',
    "imageReference": '#ImR',
    "imageSize": 'ImgS',
    "import": 'Impr',
    "impressionist": 'Imps',
    "in": 'In  ',
    "indexedColor": 'Indl',
    "indexedColorMode": 'IndC',
    "infoPaletteOptions": 'InfP',
    "infoPaletteToggleSamplers": 'InfT',
    "inherits": 'c@#^',
    "inkColors": 'InkC',
    "inkOutlines": 'InkO',
    "inkTransfer": 'InkT',
    "inks": 'Inks',
    "innerBevel": 'InrB',
    "innerGlow": 'IrGl',
    "innerGlowSource": 'glwS',
    "innerGlowSourceType": 'IGSr',
    "innerShadow": 'IrSh',
    "input": 'Inpt',
    "inputMapRange": 'Inmr',
    "inputRange": 'Inpr',
    "insetFrame": 'InsF',
    "inside": 'Insd',
    "integer": 'long',
    "integerChannel": '#inC',
    "intellectualGenre": 'ItGn',
    "intensity": 'Intn',
    "intent": 'Inte',
    "interactLevelAttr": 'inte',
    "interfaceBevelHighlight": 'IntH',
    "interfaceBevelShadow": 'Intv',
    "interfaceBlack": 'IntB',
    "interfaceBorder": 'Intd',
    "interfaceButtonDarkShadow": 'Intk',
    "interfaceButtonDownFill": 'Intt',
    "interfaceButtonUpFill": 'InBF',
    "interfaceColor": 'IClr',
    "interfaceColorBlue2": 'ICBL',
    "interfaceColorBlue32": 'ICBH',
    "interfaceColorGreen2": 'ICGL',
    "interfaceColorGreen32": 'ICGH',
    "interfaceColorRed2": 'ICRL',
    "interfaceColorRed32": 'ICRH',
    "interfaceIconFillActive": 'IntI',
    "interfaceIconFillDimmed": 'IntF',
    "interfaceIconFillSelected": 'Intc',
    "interfaceIconFrameActive": 'Intm',
    "interfaceIconFrameDimmed": 'Intr',
    "interfaceIconFrameSelected": 'IntS',
    "interfacePaletteFill": 'IntP',
    "interfaceRed": 'IntR',
    "interfaceToolTipBackground": 'IntT',
    "interfaceToolTipText": 'ITTT',
    "interfaceTransparencyBackground": 'ITBg',
    "interfaceTransparencyForeground": 'ITFg',
    "interfaceWhite": 'IntW',
    "interlace": 'Intr',
    "interlaceCreateType": 'IntC',
    "interlaceEliminateType": 'IntE',
    "interpolation": 'Intr',
    "interpolationMethod": 'IntM',
    "interpolationType": 'Intp',
    "intersect": 'Intr',
    "intersectWith": 'IntW',
    "inverse": 'Invs',
    "invert": 'Invr',
    "invertMask": 'InvM',
    "invertSource2": 'InvS',
    "invertTexture": 'InvT',
    "iptcScene": 'IpSe',
    "iptcSubjectCode": 'IpSC',
    "isDirty": 'IsDr',
    "isoCountryCode": 'IsCC',
    "itemIndex": 'ItmI',
    "javaScript": 'jsCt',
    "javaScriptMessage": 'jsMs',
    "javaScriptName": 'jsNm',
    "javaScriptOptions": 'jsop',
    "javaScriptOptionsClass": 'cjsp',
    "javaScriptText": 'jsTx',
    "justifyAll": 'JstA',
    "justifyFull": 'JstF',
    "keepProfile": 'KPro',
    "kelvin": 'Klvn',
    "kelvinCustomWhitePoint": '#Klv',
    "kernelProcessID": 'kpid',
    "kerning": 'Krng',
    "keyboardPreferences": 'KybP',
    "keyword": 'keyw',
    "keywords": 'Kywd',
    "kind": 'Knd ',
    "lab": 'Lab ',
    "lab48": 'LbCF',
    "labColor": 'LbCl',
    "labColorMode": 'LbCM',
    "labels": 'Lbls',
    "landscape": 'Lnds',
    "large": 'Lrg ',
    "largeDocumentFormat": 'Pht8',
    "last": 'Lst ',
    "lastFilter": 'LstF',
    "lastTransform": 'LstT',
    "layer": 'Lyr ',
    "layerEffects": 'Lefx',
    "layerFXVisible": 'lfxv',
    "layerID": 'LyrI',
    "layerName": 'LyrN',
    "layerOptions": 'LyrO',
    "layers": 'Lyrs',
    "layersPaletteOptions": 'LyrP',
    "leading": 'Ldng',
    "left": 'Left',
    "leftPlugin": 'Lft ',
    "length": 'Lngt',
    "lens": 'Lns ',
    "lensFlare": 'LnsF',
    "level": 'Lvl ',
    "levelBased": 'LvlB',
    "levels": 'Lvls',
    "levelsAdjustment": 'LvlA',
    "light": 'Lgt ',
    "lightBlue": 'LgtB',
    "lightDark": 'LgDr',
    "lightDirBottom": 'LDBt',
    "lightDirBottomLeft": 'LDBL',
    "lightDirBottomRight": 'LDBR',
    "lightDirLeft": 'LDLf',
    "lightDirRight": 'LDRg',
    "lightDirTop": 'LDTp',
    "lightDirTopLeft": 'LDTL',
    "lightDirTopRight": 'LDTR',
    "lightDirection": 'LghD',
    "lightDirectional": 'LghD',
    "lightGray": 'LgtG',
    "lightIntensity": 'LghI',
    "lightOmni": 'LghO',
    "lightPosBottom": 'LPBt',
    "lightPosBottomLeft": 'LPBL',
    "lightPosBottomRight": 'LPBr',
    "lightPosLeft": 'LPLf',
    "lightPosRight": 'LPRg',
    "lightPosTop": 'LPTp',
    "lightPosTopLeft": 'LPTL',
    "lightPosTopRight": 'LPTR',
    "lightPosition": 'LghP',
    "lightRed": 'LgtR',
    "lightSource": 'LghS',
    "lightSpot": 'LghS',
    "lightType": 'LghT',
    "lighten": 'Lghn',
    "lightenGrout": 'LghG',
    "lightenOnly": 'LghO',
    "lightingEffects": 'LghE',
    "lightness": 'Lght',
    "line": 'Line',
    "lineClass": 'Ln  ',
    "linear": 'Lnr ',
    "lines": 'Lns ',
    "link": 'Lnk ',
    "linked": 'Lnkd',
    "linkedLayerIDs": 'LnkL',
    "localLightingAltitude": 'Lald',
    "localLightingAngle": 'lagl',
    "localRange": 'LclR',
    "location": 'Lctn',
    "locationReference": '#Lct',
    "log": 'Log ',
    "longFloat": 'doub',
    "longInteger": 'long',
    "longLines": 'LngL',
    "longStrokes": 'LngS',
    "loopCount": 'LCnt',
    "low": 'Low ',
    "lowQuality": 'Lw  ',
    "lower": 'Lwr ',
    "lowerCase": 'LwCs',
    "luminance": 'Lmnc',
    "luminosity": 'Lmns',
    "macPaintFormat": 'McPn',
    "macThumbnail": 'McTh',
    "macintosh": 'Mcnt',
    "macintoshSystem": 'McnS',
    "magenta": 'Mgnt',
    "magentas": 'Mgnt',
    "magicEraserTool": 'MgEr',
    "magicPoint": 'Mgcp',
    "magnitude": 'magn',
    "make": 'Mk  ',
    "makeVisible": 'MkVs',
    "manipulationFOV": 'Mfov',
    "mapBlack": 'MpBl',
    "mapping": 'Mpng',
    "mappingShape": 'MpgS',
    "markdown": 'Mkdn',
    "mask": 'Msk ',
    "maskIndicator": 'MskI',
    "maskedAreas": 'MskA',
    "masterAdaptive": 'MAdp',
    "masterPerceptual": 'MPer',
    "masterSelective": 'MSel',
    "material": 'Mtrl',
    "matrix": 'Mtrx',
    "matteColor": 'MttC',
    "matteTechnique": 'BETE',
    "maximum": 'Mxm ',
    "maximumEnum": 'Mxmm',
    "maximumQuality": 'Mxm ',
    "maximumStates": 'MxmS',
    "median": 'Mdn ',
    "medium": 'Mdim',
    "mediumBlue": 'MdmB',
    "mediumDots": 'MdmD',
    "mediumLines": 'MdmL',
    "mediumQuality": 'Mdm ',
    "mediumStrokes": 'MdmS',
    "memoryPreferences": 'MmrP',
    "memoryUsagePercent": 'MmrU',
    "menuItemClass": 'Mn  ',
    "menuItemType": 'MnIt',
    "merge": 'Mrge',
    "mergeChannels": 'MrgC',
    "mergeLayers": 'MrgL',
    "mergeLayersNew": 'Mrg2',
    "mergeSpotChannel": 'MSpt',
    "mergeVisible": 'MrgV',
    "merged": 'Mrgd',
    "message": 'Msge',
    "method": 'Mthd',
    "mezzotint": 'Mztn',
    "mezzotintType": 'MztT',
    "middle": 'Mddl',
    "midpoint": 'Mdpn',
    "midtoneLevels": 'MdtL',
    "midtones": 'Mdtn',
    "millimetersUnit": '#Mlm',
    "minimum": 'Mnm ',
    "mismatchCMYK": 'MsmC',
    "mismatchGray": 'MsmG',
    "mismatchRGB": 'MsmR',
    "missedKeywordAttr": 'miss',
    "mode": 'Md  ',
    "modeGray": 'MdGr',
    "modeRGB": 'MdRG',
    "monitor": 'Moni',
    "monitorSetup": 'MntS',
    "monochromatic": 'Mnch',
    "monotone": 'Mntn',
    "mosaic": 'Msc ',
    "mosaicPlugin": 'MscT',
    "motionBlur": 'MtnB',
    "move": 'move',
    "moveTo": 'MvT ',
    "multi72Color": '72CM',
    "multi72Gray": '72GM',
    "multiNoCompositePS": 'NCmM',
    "multichannel": 'Mlth',
    "multichannelMode": 'MltC',
    "multiply": 'Mltp',
    "name": 'Nm  ',
    "navigatorPaletteOptions": 'NvgP',
    "nearestNeighbor": 'Nrst',
    "negative": 'Ngtv',
    "neonGlow": 'NGlw',
    "netscapeGray": 'NsGr',
    "neutrals": 'Ntrl',
    "neverShowDebugger": 'hnsd',
    "new": 'Nw  ',
    "newView": 'NwVw',
    "next": 'Nxt ',
    "nikon": 'Nkn ',
    "nikon105": 'Nkn1',
    "no": 'N   ',
    "noCompositePS": 'NCmp',
    "noise": 'Nose',
    "nonImageData": 'NnIm',
    "nonLinear": 'NnLn',
    "none": 'None',
    "noneUnit": '#Nne',
    "normal": 'Nrml',
    "normalPath": 'NrmP',
    "notePaper": 'NtPr',
    "notify": 'Ntfy',
    "null": 'null',
    "numLights": 'Nm l',
    "number": 'Nmbr',
    "numberOfCacheLevels": 'NCch',
    "numberOfCacheLevels64": 'NC64',
    "numberOfChannels": 'NmbO',
    "numberOfChildren": 'NmbC',
    "numberOfDocuments": 'NmbD',
    "numberOfGenerators": 'NmbG',
    "numberOfLayers": 'NmbL',
    "numberOfLevels": 'NmbL',
    "numberOfPaths": 'NmbP',
    "numberOfRipples": 'NmbR',
    "numberOfSiblings": 'NmbS',
    "object": 'Objc',
    "objectName": 'ObjN',
    "objectReference": 'obj ',
    "obsoleteTextLayer": 'TxLy',
    "oceanRipple": 'OcnR',
    "off": 'Off ',
    "offset": 'Ofst',
    "on": 'On  ',
    "onError": 'hdoe',
    "onOff": 'OnOf',
    "opacity": 'Opct',
    "opacityClass": 'Opac',
    "open": 'Opn ',
    "openAs": 'OpAs',
    "optimized": 'Optm',
    "optionalKeywordAttr": 'optk',
    "orange": 'Orng',
    "ordinal": 'Ordn',
    "orientation": 'Ornt',
    "originalAddressAttr": 'from',
    "originalHeader": 'OrgH',
    "originalTransmissionReference": 'OrgT',
    "otherCursors": 'OthC',
    "outFromCenter": 'OtFr',
    "outOfGamut": 'OtOf',
    "outerBevel": 'OtrB',
    "outerGlow": 'OrGl',
    "output": 'Otpt',
    "outsetFrame": 'OutF',
    "outside": 'Otsd',
    "overlay": 'Ovrl',
    "overprintColors": 'OvrC',
    "overrideNode": 'OvrN',
    "overrideOpen": 'OvrO',
    "overridePrinter": 'ObrP',
    "overrideSave": 'Ovrd',
    "pInherits": 'c@#^',
    "pageFormat": 'PMpf',
    "pageNumber": 'PgNm',
    "pagePosCentered": 'PgPC',
    "pagePosTopLeft": 'PgTL',
    "pagePosition": 'PgPs',
    "pageSetup": 'PgSt',
    "paint": 'Pnt ',
    "paintDaubs": 'PntD',
    "paintStroke": 'PntS',
    "paintType": 'PntT',
    "paintbrushEraser": 'Pntb',
    "paintbrushTool": 'PbTl',
    "paintingCursors": 'PntC',
    "palSecam": 'PlSc',
    "palette": 'Plt ',
    "paletteFile": 'PltF',
    "paletteKnife": 'PltK',
    "panaVision": 'PnVs',
    "paperBrightness": 'PprB',
    "parentIndex": 'PrIn',
    "parentName": 'PrNm',
    "paste": 'past',
    "pasteEffects": 'PaFX',
    "pasteInto": 'PstI',
    "pasteOutside": 'PstO',
    "patchwork": 'Ptch',
    "path": 'Path',
    "pathComponent": 'PaCm',
    "pathContents": 'PthC',
    "pathKind": 'PthK',
    "pathName": 'PthN',
    "pathPoint": 'Pthp',
    "pathReference": '#PtR',
    "pathsPaletteOptions": 'PthP',
    "pattern": 'Ptrn',
    "patternDither": 'PtnD',
    "patternKey": 'Pttn',
    "patternStampTool": 'PaTl',
    "pencilEraser": 'Pncl',
    "pencilTool": 'PcTl',
    "pencilWidth": 'Pncl',
    "percentUnit": '#Prc',
    "perceptual": 'Perc',
    "perspective": 'Prsp',
    "perspectiveIndex": 'Prsp',
    "phosphors": 'Phsp',
    "phosphorsCustomPhosphors": '#Phs',
    "photocopy": 'Phtc',
    "photoshop20Format": 'Pht2',
    "photoshop35Format": 'Pht3',
    "photoshopDCS2Format": 'PhD2',
    "photoshopDCSFormat": 'PhD1',
    "photoshopEPSFormat": 'PhtE',
    "photoshopPDFFormat": 'PhtP',
    "photoshopPicker": 'Phtk',
    "pickCMYK": 'PckC',
    "pickGray": 'PckG',
    "pickHSB": 'PckH',
    "pickLab": 'PckL',
    "pickOptions": 'PckO',
    "pickRGB": 'PckR',
    "pickerID": 'PckI',
    "pickerKind": 'Pckr',
    "pickerKindType": 'PckK',
    "pillowEmboss": 'PlEb',
    "pinch": 'Pnch',
    "pixel": 'Pxel',
    "pixelPaintFormat": 'PxlP',
    "pixelPaintSize": 'PPSz',
    "pixelPaintSize1": 'PxS1',
    "pixelPaintSize2": 'PxS2',
    "pixelPaintSize3": 'PxS3',
    "pixelPaintSize4": 'PxS4',
    "pixelsUnit": '#Pxl',
    "placeEnum": 'Plce',
    "placeEvent": 'Plc ',
    "plaster": 'Plst',
    "plasticWrap": 'PlsW',
    "platform": 'Pltf',
    "play": 'Ply ',
    "playbackOptions": 'PbkO',
    "pluginFolder": 'PlgF',
    "pluginPicker": 'PlgP',
    "pluginPrefs": 'PlgP',
    "pluginsScratchDiskPreferences": 'PlgS',
    "point": 'Pnt ',
    "point16": 'Pnt1',
    "pointillize": 'Pntl',
    "points": 'Pts ',
    "pointsUnit": '#Pnt',
    "polar": 'Plr ',
    "polarToRect": 'PlrR',
    "polygon": 'Plgn',
    "pondRipples": 'PndR',
    "position": 'Pstn',
    "postScriptColor": 'PstS',
    "posterEdges": 'PstE',
    "posterization": 'Pstr',
    "posterize": 'Pstr',
    "precise": 'Prc ',
    "preciseMatte": 'PrBL',
    "predefinedColors": 'PrdC',
    "preferBuiltin": 'PrfB',
    "preferences": 'Prfr',
    "preferencesClass": 'GnrP',
    "preserveAdditional": 'PrsA',
    "preserveLuminosity": 'PrsL',
    "preserveTransparency": 'PrsT',
    "presetKind": sTID('presetKind'),
    "presetKindCustom": sTID('presetKindCustom'),
    "presetKindDefault": sTID('presetKindDefault'),
    "presetKindType": sTID('presetKindType'),
    "pressure": 'Prs ',
    "preview": 'Prvw',
    "previewBlack": 'PrvB',
    "previewCMY": 'PrvN',
    "previewCMYK": 'PrvK',
    "previewCMYKEnum": 'PrvC',
    "previewCMYKType": 'Prvt',
    "previewCyan": 'Prvy',
    "previewFullSize": 'PrvF',
    "previewIcon": 'PrvI',
    "previewMacThumbnail": 'PrvM',
    "previewMagenta": 'PrvM',
    "previewOff": 'PrvO',
    "previewWinThumbnail": 'PrvW',
    "previewYellow": 'PrvY',
    "previewsQuery": 'PrvQ',
    "previous": 'Prvs',
    "primaries": 'Prim',
    "print": 'Prnt',
    "printSettings": 'PMps',
    "printSize": 'PrnS',
    "printingInksSetup": 'PrnI',
    "processSerialNumber": 'psn ',
    "profileMismatch": 'PrfM',
    "profileSetup": 'PrfS',
    "profileToProfile": 'PrfT',
    "property": 'Prpr',
    "proportionalHeight": 'HiPr',
    "proportionalWidth": 'WiPr',
    "provinceState": 'PrvS',
    "purge": 'Prge',
    "purgeItem": 'PrgI',
    "purple": 'Prp ',
    "pyramids": 'Pyrm',
    "quadCenterState": 'QCSt',
    "quadtone": 'Qdtn',
    "quality": 'Qlty',
    "queryAlways": 'QurA',
    "queryAsk": 'Qurl',
    "queryNever": 'QurN',
    "queryState": 'QurS',
    "quickMask": 'QucM',
    "quit": 'quit',
    "radial": 'Rdl ',
    "radialBlur": 'RdlB',
    "radius": 'Rds ',
    "random": 'Rndm',
    "randomSeed": 'RndS',
    "range": 'Rang',
    "rasterize": 'Rstr',
    "rasterizeTypeLayer": 'RstT',
    "ratio": 'Rt  ',
    "rawData": 'tdta',
    "rawFormat": 'Rw  ',
    "recentFiles": 'Rcnf',
    "rect16": 'Rct1',
    "rectToPolar": 'RctP',
    "rectangle": 'Rctn',
    "red": 'Rd  ',
    "redBlackPoint": 'RdBl',
    "redGamma": 'RdGm',
    "redWhitePoint": 'RdWh',
    "redX": 'RdX ',
    "redY": 'RdY ',
    "redrawComplete": 'RdCm',
    "reds": 'Rds ',
    "reflected": 'Rflc',
    "registrationMarks": 'RgsM',
    "relative": 'Rltv',
    "relief": 'Rlf ',
    "removeBlackMatte": 'RmvB',
    "removeLayerMask": 'RmvL',
    "removeWhiteMatte": 'RmvW',
    "rename": 'Rnm ',
    "renderFidelity": 'Rfid',
    "repeat": 'Rpt ',
    "repeatEdgePixels": 'RptE',
    "replaceColor": 'RplC',
    "resample": 'Rsmp',
    "reset": 'Rset',
    "resizeWindowsOnZoom": 'RWOZ',
    "resolution": 'Rslt',
    "resourceID": 'RsrI',
    "response": 'Rspn',
    "retainHeader": 'RtnH',
    "reticulation": 'Rtcl',
    "returnIDAttr": 'rtid',
    "revealAll": 'RvlA',
    "revealSelection": 'RvlS',
    "reverse": 'Rvrs',
    "revert": 'Rvrt',
    "right": 'Rght',
    "rightsUsage": 'RiUs',
    "ripple": 'Rple',
    "rippleMagnitude": 'RplM',
    "rippleSize": 'RplS',
    "rotate": 'Rtt ',
    "rotateEventEnum": 'Rtte',
    "rotoscopingPreferences": 'RtsP',
    "roughPastels": 'RghP',
    "round": 'Rnd ',
    "roundness": 'Rndn',
    "rulerCm": 'RrCm',
    "rulerInches": 'RrIn',
    "rulerOriginH": 'RlrH',
    "rulerOriginV": 'RlrV',
    "rulerPercent": 'RrPr',
    "rulerPicas": 'RrPi',
    "rulerPixels": 'RrPx',
    "rulerPoints": 'RrPt',
    "rulerUnits": 'RlrU',
    "sInt16": 'shor',
    "sInt32": 'long',
    "sInt64": 'comp',
    "sMFloat": 'sing',
    "sMInt": 'shor',
    "sRGB": 'SRGB',
    "sample3x3": 'Smp3',
    "sample5x5": 'Smp5',
    "samplePoint": 'SmpP',
    "saturate": 'Str ',
    "saturation": 'Strt',
    "saturationTool": 'SrTl',
    "save": 'save',
    "saveAndClose": 'SvAn',
    "saveComposite": 'SvCm',
    "saveForWeb": 'Svfw',
    "savePaletteLocations": 'PltL',
    "savePaths": 'SvPt',
    "savePyramids": 'SvPy',
    "saved": 'Sved',
    "saveStage": stringIDToTypeID("saveStage"),
    "saveStageType": stringIDToTypeID("saveStageType"),
    "saveBegin": stringIDToTypeID("saveBegin"),
    "saveSucceeded": stringIDToTypeID("saveSucceeded"),
    "saving": 'Svng',
    "savingFilesPreferences": 'SvnF',
    "scale": 'Scl ',
    "scaleHorizontal": 'SclH',
    "scaleVertical": 'SclV',
    "scaling": 'Scln',
    "scans": 'Scns',
    "scitexCTFormat": 'Sctx',
    "scratchDisks": 'ScrD',
    "screen": 'Scrn',
    "screenCircle": 'ScrC',
    "screenDot": 'ScrD',
    "screenFile": 'ScrF',
    "screenLine": 'ScrL',
    "screenType": 'ScrT',
    "sectionH": 'sect',
    "select": 'slct',
    "selectedAreas": 'SlcA',
    "selection": 'fsel',
    "selectionClass": 'csel',
    "selectionEnum": 'Slct',
    "selective": 'Sele',
    "selectiveColor": 'SlcC',
    "separationSetup": 'SprS',
    "separationTables": 'SprT',
    "separations": 'Sprt',
    "serialString": 'SrlS',
    "sessionID": 'ssid',
    "set": 'setd',
    "shadingIntensity": 'ShdI',
    "shadingNoise": 'ShdN',
    "shadingShape": 'ShdS',
    "shadowColor": 'sdwC',
    "shadowIntensity": 'ShdI',
    "shadowLevels": 'ShdL',
    "shadowMode": 'sdwM',
    "shadowOpacity": 'sdwO',
    "shadows": 'Shdw',
    "shape": 'Shp ',
    "shapeCurveCustom": 'sp06',
    "shapeCurveDouble": 'sp04',
    "shapeCurveGaussian": 'sp02',
    "shapeCurveLinear": 'sp01',
    "shapeCurveSingle": 'sp03',
    "shapeCurveTriple": 'sp05',
    "shapeCurveType": 'ShpC',
    "shapingCurve": 'ShpC',
    "sharpen": 'Shrp',
    "sharpenEdges": 'ShrE',
    "sharpenMore": 'ShrM',
    "sharpenTool": 'ShTl',
    "sharpness": 'Shrp',
    "shear": 'Shr ',
    "shearEd": 'ShrE',
    "shearPoints": 'ShrP',
    "shearSt": 'ShrS',
    "shiftKey": 'ShfK',
    "shiftKeyToolSwitch": 'ShKT',
    "shortFloat": 'sing',
    "shortInteger": 'shor',
    "shortLines": 'ShrL',
    "shortNames": 'ShrN',
    "shortStrokes": 'ShSt',
    "show": 'Shw ',
    "showEnglishFontNames": 'ShwE',
    "showToolTips": 'ShwT',
    "showTransparency": 'ShTr',
    "similar": 'Smlr',
    "single72Color": '72CS',
    "single72Gray": '72GS',
    "singleColumn": 'Sngc',
    "singleNoCompositePS": 'NCmS',
    "singleRow": 'Sngr',
    "size": 'Sz  ',
    "skew": 'Skew',
    "slopeLimitMatte": 'Slmt',
    "slot": 'slot',
    "small": 'Sml ',
    "smartBlur": 'SmrB',
    "smartBlurMode": 'SmBM',
    "smartBlurModeEdgeOnly": 'SBME',
    "smartBlurModeNormal": 'SBMN',
    "smartBlurModeOverlayEdge": 'SBMO',
    "smartBlurQuality": 'SmBQ',
    "smartBlurQualityHigh": 'SBQH',
    "smartBlurQualityLow": 'SBQL',
    "smartBlurQualityMedium": 'SBQM',
    "smartBrushClearSelectionEvent": 'SBCs',
    "smartBrushResetSelectionEvent": 'SBRs',
    "smooth": 'Smoo',
    "smoothness": 'Smth',
    "smudgeStick": 'SmdS',
    "smudgeTool": 'SmTl',
    "snapshotClass": 'SnpS',
    "snapshotEnum": 'Snps',
    "snapshotInitial": 'SnpI',
    "softLight": 'SftL',
    "softMatte": 'SfBL',
    "softness": 'Sftn',
    "solarize": 'Slrz',
    "solidColor": 'SClr',
    "solidFill": 'SoFi',
    "source": 'Srce',
    "source2": 'Src2',
    "sourceMode": 'SrcM',
    "sourceModeType": 'Cndn',
    "spacing": 'Spcn',
    "spatter": 'Spt ',
    "specialInstructions": 'SpcI',
    "spectrum": 'Spct',
    "spherical": 'Sph ',
    "spherize": 'Sphr',
    "spherizeMode": 'SphM',
    "spin": 'Spn ',
    "splitChannels": 'SplC',
    "sponge": 'Spng',
    "spot": 'Spot',
    "spotColor": 'Spot',
    "spotColorChannel": 'SCch',
    "sprayRadius": 'SprR',
    "sprayedStrokes": 'SprS',
    "square": 'Sqr ',
    "squareSize": 'SqrS',
    "srcBlackMax": 'Srcl',
    "srcBlackMin": 'SrcB',
    "srcWhiteMax": 'Srcm',
    "srcWhiteMin": 'SrcW',
    "stagger": 'Stgr',
    "stainedGlass": 'StnG',
    "stamp": 'Stmp',
    "stampIn": 'In  ',
    "stampOut": 'Out ',
    "standard": 'Std ',
    "start": 'Strt',
    "startArrowhead": 'StrA',
    "state": 'Stte',
    "stdA": 'StdA',
    "stdB": 'StdB',
    "stdC": 'StdC',
    "stdE": 'StdE',
    "stop": 'Stop',
    "strength": 'srgh',
    "strengthPlugin": 'Strg',
    "strengthRatio": 'srgR',
    "stretchToFit": 'StrF',
    "stringChannel": '#sth',
    "stringClassFormat": '#StC',
    "stringCompensation": '#Stm',
    "stringFSS": '#Stf',
    "stringInteger": '#StI',
    "stroke": 'Strk',
    "strokeDetail": 'StDt',
    "strokeDirHorizontal": 'SDHz',
    "strokeDirLeftDiag": 'SDLD',
    "strokeDirRightDiag": 'SDRD',
    "strokeDirVertical": 'SDVt',
    "strokeDirection": 'SDir',
    "strokeDirectionType": 'StrD',
    "strokeLength": 'StrL',
    "strokeLocation": 'StrL',
    "strokePressure": 'StrP',
    "strokeSize": 'StrS',
    "strokeWidth": 'StrW',
    "style": 'Styl',
    "styleClass": 'StyC',
    "styles": 'Stys',
    "stylesAppend": 'SlsA',
    "stylesDelete": 'Slsf',
    "stylesLoad": 'Slsd',
    "stylesNew": 'SlsN',
    "stylesReset": 'SlsR',
    "stylesSave": 'Slsv',
    "stylusIsColor": 'StlC',
    "stylusIsOpacity": 'StlO',
    "stylusIsPressure": 'StlP',
    "stylusIsSize": 'StlS',
    "subpath": 'Sbpt',
    "subpathListKey": 'SbpL',
    "subpathsList": 'Sbpl',
    "subtract": 'Sbtr',
    "subtractFrom": 'SbtF',
    "sumie": 'Smie',
    "supplementalCategories": 'SplC',
    "swatchesAppend": 'SwtA',
    "swatchesReplace": 'Swtp',
    "swatchesReset": 'SwtR',
    "swatchesSave": 'SwtS',
    "systemInfo": 'SstI',
    "systemPalette": 'SstP',
    "systemPicker": 'SysP',
    "tables": 'Tbl ',
    "takeMergedSnapshot": 'TkMr',
    "takeSnapshot": 'TkSn',
    "targaFormat": 'TrgF',
    "target": cTID('Trgt'),
    "targetEnum": 'Trgt',
    "targetID": 'targ',
    "targetPath": 'Trgp',
    "targetPathIndex": 'TrgP',
    "texTypeBlocks": 'TxBl',
    "texTypeBrick": 'TxBr',
    "texTypeBurlap": 'TxBu',
    "texTypeCanvas": 'TxCa',
    "texTypeFrosted": 'TxFr',
    "texTypeSandstone": 'TxSt',
    "texTypeTinyLens": 'TxTL',
    "textClickPoint": 'TxtC',
    "textData": 'TxtD',
    "textKey": 'Txt ',
    "textLayer": 'TxLr',
    "textStyle": 'TxtS',
    "textStyleRange": 'Txtt',
    "textType": 'TEXT',
    "texture": 'Txtr',
    "textureCoverage": 'TxtC',
    "textureFile": 'TxtF',
    "textureFill": 'TxtF',
    "textureType": 'TxtT',
    "texturizer": 'Txtz',
    "threshold": 'Thsh',
    "thresholdClassEvent": 'Thrs',
    "thresholdEnum": 'Thrh',
    "thumbnail": 'Thmb',
    "tile": 'Tile',
    "tileNumber": 'TlNm',
    "tileOffset": 'TlOf',
    "tilePlugin": 'Tl  ',
    "tileSize": 'TlSz',
    "tiles": 'Tls ',
    "timeoutAttr": 'timo',
    "title": 'Ttl ',
    "to": 'T   ',
    "toBuiltin": 'TBl ',
    "toLinked": 'ToLk',
    "toMode": 'TMd ',
    "toggleActionsPalette": 'TglA',
    "toggleAnnots": 'TgAn',
    "toggleBlackPreview": 'TgBP',
    "toggleBrushesPalette": 'TglB',
    "toggleCMYKPreview": 'TglC',
    "toggleCMYPreview": 'TgCM',
    "toggleChannelsPalette": 'Tglh',
    "toggleColorPalette": 'Tglc',
    "toggleCyanPreview": 'TgCP',
    "toggleEdges": 'TglE',
    "toggleGamutWarning": 'TglG',
    "toggleGrid": 'TgGr',
    "toggleGuides": 'Tgld',
    "toggleHistoryPalette": 'TglH',
    "toggleInfoPalette": 'TglI',
    "toggleLayerMask": 'TglM',
    "toggleLayersPalette": 'Tgly',
    "toggleLockGuides": 'TglL',
    "toggleMagentaPreview": 'TgMP',
    "toggleNavigatorPalette": 'TglN',
    "toggleOptionsPalette": 'TglO',
    "toggleOthers": 'TglO',
    "togglePaths": 'TglP',
    "togglePathsPalette": 'Tglt',
    "toggleRGBMacPreview": 'TrMp',
    "toggleRGBUncompensatedPreview": 'TrUp',
    "toggleRGBWindowsPreview": 'TrWp',
    "toggleRulers": 'TglR',
    "toggleSnapToGrid": 'TgSn',
    "toggleSnapToGuides": 'TglS',
    "toggleStatusBar": 'Tgls',
    "toggleStylesPalette": 'TgSl',
    "toggleSwatchesPalette": 'Tglw',
    "toggleToolsPalette": 'TglT',
    "toggleYellowPreview": 'TgYP',
    "tolerance": 'Tlrn',
    "tool": 'Tool',
    "top": 'Top ',
    "tornEdges": 'TrnE',
    "totalLimit": 'TtlL',
    "traceContour": 'TrcC',
    "tracking": 'Trck',
    "transactionIDAttr": 'tran',
    "transferFunction": 'TrnF',
    "transferPoint": 'DtnP',
    "transferSpec": 'TrnS',
    "transferSpecClass": 'Trfp',
    "transform": 'Trnf',
    "transparency": 'Trns',
    "transparencyEnum": 'Trsp',
    "transparencyGamutPreferences": 'TrnG',
    "transparencyGrid": 'TrnG',
    "transparencyGridColors": 'TrnC',
    "transparencyGridColorsType": 'Trnl',
    "transparencyGridSize": 'TrnG',
    "transparencyPrefs": 'TrnP',
    "transparencyShape": 'TrnS',
    "transparencyStop": 'TrnS',
    "transparent": 'Trns',
    "transparentIndex": 'TrnI',
    "transparentWhites": 'TrnW',
    "trap": 'Trap',
    "trinitron": 'Trnt',
    "tritone": 'Trtn',
    "true": 'true',
    "twirl": 'Twrl',
    "twist": 'Twst',
    "type": 'Type',
    "typeClassModeOrClassMode": '#TyM',
    "typeFrameDispose": 'FrmD',
    "uIBitmap": 'UBtm',
    "uICMYK": 'UCMY',
    "uIDuotone": 'UDtn',
    "uIGrayscale": 'UGry',
    "uIIndexed": 'UInd',
    "uILab": 'ULab',
    "uIMultichannel": 'UMlt',
    "uIRGB": 'URGB',
    "uInt32": 'magn',
    "uiBlueHighlightColor": 'UIHb',
    "uiDefaultHighlightColor": 'UIHd',
    "undefinedArea": 'UndA',
    "underline": 'Undl',
    "underpainting": 'Undr',
    "undoEnum": 'Und ',
    "undoEvent": 'undo',
    "ungroup": 'Ungr',
    "uniform": 'Unfm',
    "uniformDistribution": 'Unfr',
    "unitsPrefs": 'UntP',
    "unitsRulersPreferences": 'UntR',
    "unlink": 'Unlk',
    "unsharpMask": 'UnsM',
    "unspecifiedColor": 'UnsC',
    "untitled": 'Untl',
    "upper": 'Upr ',
    "upperY": 'UppY',
    "urgency": 'Urgn',
    "useAccurateScreens": 'AcrS',
    "useAdditionalPlugins": 'AdPl',
    "useCacheForHistograms": 'UsCc',
    "useCurves": 'UsCr',
    "useDefault": 'UsDf',
    "useGlobalAngle": 'uglg',
    "useICCProfile": 'UsIC',
    "useMask": 'UsMs',
    "userMaskEnabled": 'UsrM',
    "userMaskLinked": 'Usrs',
    "userMaskOptions": 'UsrM',
    "userStop": 'UsrS',
    "using": 'Usng',
    "vMPreferences": 'VMPr',
    "value": 'Vl  ',
    "valueList": 'VlLs',
    "variations": 'Vrtn',
    "vector0": 'Vct0',
    "vector1": 'Vct1',
    "vectorColor": 'VctC',
    "version": 'Vrsn',
    "versionFix": 'VrsF',
    "versionMajor": 'VrsM',
    "versionMinor": 'VrsN',
    "vertical": 'Vrtc',
    "verticalLocation": 'VrtL',
    "verticalOnly": 'VrtO',
    "verticalScale": 'VrtS',
    "videoAlpha": 'Vdlp',
    "violet": 'Vlt ',
    "visible": 'Vsbl',
    "wait": 'Wait',
    "watchSuspension": 'WtcS',
    "waterPaper": 'WtrP',
    "watercolor": 'Wtrc',
    "watermark": 'watr',
    "wave": 'Wave',
    "waveSine": 'WvSn',
    "waveSquare": 'WvSq',
    "waveTriangle": 'WvTr',
    "waveType": 'Wvtp',
    "wavelengthMax": 'WLMx',
    "wavelengthMin": 'WLMn',
    "web": 'Web ',
    "wetBrushTool": 'MixB',
    "wetEdges": 'Wtdg',
    "what": 'What',
    "white": 'Wht ',
    "whiteClip": 'WhtC',
    "whiteIntensity": 'WhtI',
    "whiteIsHigh": 'WhHi',
    "whiteLevel": 'WhtL',
    "whitePoint": 'WhtP',
    "whites": 'Whts',
    "wholePath": 'WhPt',
    "wideGamutRGB": 'WRGB',
    "widePhosphors": 'Wide',
    "width": 'Wdth',
    "widthUnit": 'WidU',
    "wildCard": '****',
    "winThumbnail": 'WnTh',
    "wind": 'Wnd ',
    "windMethod": 'WndM',
    "windows": 'Win ',
    "windowsSystem": 'WndS',
    "with": 'With',
    "workPath": 'WrPt',
    "workPathIndex": 'WrkP',
    "wrap": 'Wrp ',
    "wrapAround": 'WrpA',
    "x": 'X   ',
    "xYYColor": 'XYYC',
    "y": 'Y   ',
    "yellow": 'Yllw',
    "yellowColor": 'Ylw ',
    "yellows": 'Ylws',
    "yes": 'Ys  ',
    "yesNo": 'YsN ',
    "zigZag": 'ZgZg',
    "zigZagType": 'ZZTy',
    "zip": 'ZpEn',
    "zoom": 'Zm  ',
    "zoomIn": 'ZmIn',
    "zoomOut": 'ZmOt'
};

try {
    for (var key in Codes) {
        var charId = Codes[key];
        if (typeof charId === "string") {
            var typeId = cTID(charId);
            Codes[key] = typeId;
        }
    }
} catch (err) {
    alert("Error at Codes.jsx: " + err);
}

// ------------ Usings ------------------------ //

function usingDialogMode(dialogMode, callback, throwErrors) {

    var resetValue = app.displayDialogs;

    try {
        app.displayDialogs = dialogMode;
        callback();
    } catch (err) {
        if (throwErrors) {
            throw err;
        } else {
            alert("usingDialogMode: " + err);
        }
    } finally {
        app.displayDialogs = resetValue;
    }
}

function usingLayer(layer, callback) {
    var returnValue = undefined;
    var document = layer.parent;
    var userActiveLayer = document.activeLayer;
    var exception = undefined;
    try {
        document.activeLayer = layer;
        returnValue = callback();
    } catch (err) {
        $.writeln("Error at Beautypanel/ArtLayers.jsx usingLayer: " + err);
        exception = err;
    } finally {
        document.activeLayer = userActiveLayer;
        if (exception) {
            throw exception;
        }
    }
    return returnValue;
}

function usingDocument(doc, action) {
    try {
        var returnValue = undefined;
        var originalDoc = app.activeDocument;
        app.activeDocument = doc;
        returnValue = action();
        app.activeDocument = originalDoc;
        return returnValue;
    } catch (err) {
        alert("Error at usingDocument(): " + err);
    }
}

// ------------ Brush functions --------------- //

function getBrushOpacity() {
    var ref = new ActionReference();
    ref.putEnumerated(Codes.application, Codes.ordinal, Codes.target);
    var desc = executeActionGet(ref);
    desc = desc.getObjectValue(Codes.currentToolOptions);
    return desc.getInteger(Codes.opacity);
}

function convertDrawingMode(mode) {
    // =======================================================
    var idCnvM = charIDToTypeID(Codes.convertMode);
    var desc429 = new ActionDescriptor();
    var idT = charIDToTypeID(Codes.to);
    var idLbCM = charIDToTypeID(mode);
    desc429.putClass(idT, idLbCM);
    executeAction(idCnvM, desc429, DialogModes.NO);

}

// ------------ Application functions --------------- //

function convertTo16BitMode() {
    var desc = new ActionDescriptor();
    desc.putInteger(Codes.depth, 16);
    desc.putBoolean(Codes.merge, false);
    executeAction(Codes.convertMode, desc, DialogModes.NO);
}

function toChangeMode(documentMode) {

    switch (documentMode) {
        case DocumentMode.BITMAP:
            return ChangeMode.BITMAP;
        case DocumentMode.CMYK:
            return ChangeMode.CMYK;
        case DocumentMode.GRAYSCALE:
            return ChangeMode.GRAYSCALE;
        case DocumentMode.INDEXEDCOLOR:
            return ChangeMode.INDEXEDCOLOR;
        case DocumentMode.LAB:
            return ChangeMode.LAB;
        case DocumentMode.MULTICHANNEL:
            return ChangeMode.MULTICHANNEL;
        case DocumentMode.RGB:
            return ChangeMode.RGB;
    }

    return null;
}

function runMenuItem(menuItem) {
    var desc = new ActionDescriptor();
    var ref = new ActionReference();
    ref.putEnumerated(Codes.menuItemClass, Codes.menuItemType, menuItem);
    desc.putReference(Codes.target, ref);
    executeAction(Codes.select, desc, DialogModes.ERROR);
}

function getZoomLevel() {
    var ref = new ActionReference();
    ref.putProperty(stringIDToTypeID('property'), stringIDToTypeID('zoom'));
    ref.putEnumerated(charIDToTypeID("Dcmn"), charIDToTypeID("Ordn"), charIDToTypeID("Trgt"));
    var desc = executeActionGet(ref);
    return Number(desc.getDouble(stringIDToTypeID('zoom')) * 100).toFixed(1);
};

// =========== Drawing ============ //

function markHistoryPosition() {
    var _mark = app.activeDocument.activeHistoryState;
    return function () {
        app.activeDocument.activeHistoryState = _mark;
    }
}

// =========== FileSystem ============ //

function deleteFolder(path) {
    try {

        var folder = new Folder();

        if (folder.exists) {

            var deleted = folder.remove();

            if (!deleted) {
                throw new Error("Folder delete failed. Unknown error.");
            }

        }

    } catch (err) {
        throw new Error("Folder delete failed. " + err);
    }

}
function mkdir(path) {

    var folder = new Folder(path);

    if (!folder.exists) {
        var parts = path.split('/');
        parts.pop();
        mkdir(parts.join('/'));
        folder.create();
    }
}
function copyFolder(from, to, overwrite) {

    if (typeof overwrite !== "boolean")
        overwrite = false;

    var files = from.getFiles();

    for (var i = 0; i < files.length; i++) {

        var f = files[i];

        if (f instanceof Folder) {
            copyFolder(f, to + "/" + f.name, overwrite);
        } else if (f instanceof File) {
            copyFile(f, new File(to + "/" + f.name), overwrite);
        }
    }

}
function copyFile(from, to, overwrite) {

    if (typeof overwrite !== "boolean")
        overwrite = false;

    if (!from.exists) {
        return;
    }

    if (to.exists) {
        if (!overwrite)
            return;

        to.remove();
    }
    if (!to.parent.exists) {
        to.parent.create();
    }

    from.copy(to);

}

// ======= Beautypanel (Because can't include here) ================

var Beautypanel = {

    layers: {
        soft: null,
        detail: null,
        levels: null,
        detailBlackWhite: null,
        detailColor: null,
        detailEnhance: null,
        enhanceDetails: null,
        dodgeAndBurnGray: null,
        orthon: null,
        vignette: null,
        dark: null,
        bright: null,
        autumn: null,
        referenceLayer: null
    },

    log: (function () {

        var fileDescriptors = {};

        return function (message, stage) {

            try {

                if (!stage) {
                    stage = "log";
                }

                var folder, file;

                if (stage in fileDescriptors) {

                    file = fileDescriptors[stage];

                } else {
                    folder = BEAUTYPANEL.folders.appData + "/log";
                    file = folder + "/" + stage + ".log";

                    folder = new Folder(folder)
                    if (!folder.exists) {
                        folder.create();
                    }

                    file = new File(file);
                    fileDescriptors[stage] = file;
                }

                file.open("a");
                file.writeln(message);
                file.close();

            } catch (err) {
                alert("Log failed. " + err)
            }

        }
    })(),

    LayerUtils: {
        applySaturation: function (layer) {
            return usingLayer(layer || this, function () {
                return app.executeAction(Codes.desaturate, undefined, DialogModes.ERROR);
            });
        },
        applyCalculation: function (layer, targetLayer, channel, invert, mode, scale, offset) {

            return usingLayer(layer || this, function () {

                // === Check parameters

                if (typeof channel === "string") {
                    channel = charIDToTypeID(channel);
                }
                if (typeof mode === "string") {
                    mode = charIDToTypeID(mode);
                }
                if (typeof invert !== "boolean") {
                    invert = false;
                }
                if (typeof scale !== "number") {
                    scale = 1;
                }
                if (typeof offset !== "number") {
                    offset = 0;
                }

                // ====================================================
                var actionDescriptpr = new ActionDescriptor();
                var parameterDescriptor = new ActionDescriptor();
                var reference = new ActionReference();
                reference.putEnumerated(Codes.channel, Codes.channel, channel);
                reference.putName(Codes.layer, targetLayer.name);
                parameterDescriptor.putReference(Codes.to, reference);
                parameterDescriptor.putBoolean(Codes.invert, true);
                parameterDescriptor.putEnumerated(Codes.calculation, Codes.calculation, mode);
                parameterDescriptor.putDouble(Codes.scale, 2.000000);
                parameterDescriptor.putInteger(Codes.offset, 0);
                actionDescriptpr.putObject(Codes.with, Codes.calculation, parameterDescriptor);

                return app.executeAction(Codes.applyImageEvent, actionDescriptpr, DialogModes.NO);

            });
        },
        createContrastLayer: function (layer, rangeStart, rangeEnd) {

            return usingLayer(layer, function () {

                // =======================================================
                // Create Adjustment Layer

                var desc306 = new ActionDescriptor();
                var ref142 = new ActionReference();
                ref142.putClass(Codes.adjustmentLayer);
                desc306.putReference(Codes.null, ref142);
                var desc307 = new ActionDescriptor();
                var desc308 = new ActionDescriptor();
                desc308.putEnumerated(Codes.presetKind, Codes.presetKindType, Codes.presetKindDefault);
                desc307.putObject(Codes.type, Codes.levels, desc308);
                desc306.putObject(Codes.using, Codes.adjustmentLayer, desc307);
                executeAction(Codes.make, desc306, DialogModes.ERROR);

                // =======================================================
                // Set Curves for Adjustment Layer

                var desc251 = new ActionDescriptor();

                var ref112 = new ActionReference();
                ref112.putEnumerated(Codes.adjustmentLayer, Codes.ordinal, Codes.target);
                desc251.putReference(Codes.null, ref112);

                var desc252 = new ActionDescriptor();
                desc252.putEnumerated(Codes.presetKind, Codes.presetKindType, Codes.presetKindCustom);

                var list50 = new ActionList();
                var desc253 = new ActionDescriptor();
                var ref113 = new ActionReference();
                ref113.putEnumerated(Codes.channel, Codes.channel, Codes.composite);
                desc253.putReference(Codes.channel, ref113);

                var list51 = new ActionList();
                list51.putInteger(rangeStart);
                list51.putInteger(rangeEnd);
                desc253.putList(Codes.input, list51);

                list50.putObject(Codes.levelsAdjustment, desc253);
                desc252.putList(Codes.adjustment, list50);

                desc251.putObject(Codes.to, Codes.levels, desc252);
                executeAction(Codes.set, desc251, DialogModes.ERROR);

                return layer.parent.activeLayer;

            })

        },
        createRvlaMask: function (layer) {

            return usingLayer(layer || this, function () {

                var reference = new ActionReference();
                reference.putEnumerated(Codes.channel, Codes.channel, Codes.mask);

                var desc = new ActionDescriptor();
                desc.putReference(Codes.at, reference);
                desc.putClass(Codes.new, Codes.channel);
                desc.putEnumerated(Codes.using, Codes.userMaskEnabled, Codes.revealAll);

                return app.executeAction(Codes.make, desc, DialogModes.ERROR);

            })

        },
        applySurfaceBlur: function (layer, radius, threshold, dialogMode) {
            return usingLayer(layer || this, function () {
                var descriptor = new ActionDescriptor();
                descriptor.putUnitDouble(Codes.radius, Codes.pixelsUnit, radius);
                descriptor.putInteger(Codes.threshold, threshold);
                return app.executeAction(stringIDToTypeID("surfaceBlur"), descriptor, dialogMode || app.displayDialogs);
            });
        }
    },

    DocumentUtils: {
        getLayerByName: function (document, name) {
            try {
                if (typeof document === "string") {
                    name = document;
                    document = app.activeDocument;
                }
                return document.layers.getByName(name);
            } catch (err) {
                // Dummy error. Photoshop throws an error if layer does not exist. We simply return null.
                return null;
            }
        },
        mergeLayers: function (document, layers) {

            try {

                if (layers.length === 0) {
                    throw "No layers to merge applied.";
                }

                var args = arguments;

                if (!layers[0]) {
                    throw "Layer at index 0 is null.";
                }

                document.activeLayer = layers[0];
                var name = document.activeLayer.name;

                usingDocument(document, function () {

                    // Select layers to merge
                    for (var i = 0; i < layers.length; i++) {

                        var layer = layers[i];

                        if (!layer) {
                            throw "Layer at index " + i + " is null.";
                        }

                        var desc = new ActionDescriptor();
                        var ref = new ActionReference();
                        ref.putName(Codes.layer, layer.name);
                        desc.putReference(Codes.null, ref);
                        desc.putEnumerated(stringIDToTypeID("selectionModifier"), stringIDToTypeID("selectionModifierType"), stringIDToTypeID("addToSelection"));
                        desc.putBoolean(Codes.makeVisible, false);

                        app.executeAction(Codes.select, desc, DialogModes.NO);

                    }

                    // Do the merge
                    app.executeAction(cTID("Mrg2"), new ActionDescriptor(), DialogModes.NO);

                });

                return document.layers.getByName(name);

            } catch (err) {
                alert("Error at $.DocumentUtils.mergeLayers(): " + err);
                return null;
            }
        }
    },

    checkBackgroundLayer: function () {
        try {

            var document = app.activeDocument;

            if (document) {

                // ======================================== //
                // Check for background layer
                var backgroundLayer = null;
                for (var i = 0; i < document.layers.length; i++) {
                    if (document.layers[i].isBackgroundLayer) {
                        backgroundLayer = document.layers[i];
                    }
                }

                // ======================================== //
                // If no backgroundLayer ask user to set first layer as background layer
                if (!backgroundLayer) {
                    var confirmed = confirm(strings.request_no_background_layer);
                    if (confirmed) {
                        document.layers[0].isBackgroundLayer = true;
                    }
                }

            }

        } catch (err) {
            alert("Error at Beautypanel.checkBackgroundLayer(): " + err);
        }

    },
    checkBitsPerChannel: function () {

        // ===================================================== //
        // Convert document to 16-Bit color mode if current is 8-Bit or lower.
        // Convert only if user confirmed request.
        // ===================================================== //

        try {

            // ===================================================== //
            switch (app.activeDocument.bitsPerChannel) {

                case BitsPerChannelType.ONE:
                case BitsPerChannelType.EIGHT:

                    // ===================================================== //
                    // Ask user for permission to convert document to 16-Bit color mode
                    var convert = confirm(strings.request_convert_bits_per_channel);

                    // ===================================================== //
                    // If user agreed, convert document to 16-Bit color mode
                    if (convert) {
                        app.activeDocument.bitsPerChannel = BitsPerChannelType.SIXTEEN;
                    }

                    break;

                default:
                    return;

            }

        } catch (err) {
            alert("Beautypanel.checkBitsPerChannel: " + err);
        }

    },
    refreshLayers: function () {

        var document = app.activeDocument;

        var gL = function (name) {
            return Beautypanel.DocumentUtils.getLayerByName(document, strings["layername_" + name]);
        };

        Beautypanel.layers.soft = gL("soft");
        Beautypanel.layers.detail = gL("detail");
        Beautypanel.layers.detailBlackWhite = gL("detail_bw");
        Beautypanel.layers.detailColor = gL("detail_color");
        Beautypanel.layers.detailEnhance = gL("detail_enhance");
        Beautypanel.layers.levels = gL("levels");
        Beautypanel.layers.enhanceDetails = gL("enhancedDetails");
        Beautypanel.layers.dodgeAndBurnGray = gL("dodgeAndBurnGray");
        Beautypanel.layers.orthon = gL("orthon");
        Beautypanel.layers.vignette = gL("vignette");
        Beautypanel.layers.dark = gL("dark");
        Beautypanel.layers.bright = gL("bright");
        Beautypanel.layers.autumn = gL("autumn");

    },

    // Tools
    frequencySeparation: null,
    dodgeAndBurnGray: null,
    dodgeAndBurnGradient: null,

    // Sharpen
    unsharpMask: function (amount, radius, threshold, switchToDetail, withDialog) {

        var returnValue = undefined;

        try {

            // ===================================================================== //
            // Check arguments

            var dialogMode = DialogModes.ALL;
            var document = app.activeDocument;
            var gc = BEAUTYPANEL.config.unsharpMask;

            var usedGlobalConfig = false;

            if (typeof amount !== "number") {
                amount = gc.amount;
            }

            if (typeof radius !== "number") {
                radius = gc.radius;
            }

            if (typeof threshold !== "number") {
                threshold = gc.threshold;
            }

            if (typeof switchToDetail !== "boolean") {
                switchToDetail = gc.switchToDetail;
            }

            if (typeof withDialog === "boolean") {
                dialogMode = withDialog ? DialogModes.ALL : DialogModes.ERROR;
            }

            // ===================================================================== //
            // If checkbox "Switch to Detail" is checked and if details layer exist, switch to detail.
            if (switchToDetail && Beautypanel.layers.detail) {
                document.activeLayer = Beautypanel.layers.detail;
				}

            // ===================================================================== //
            // If we are on soft layer, ask user for permission to switch to details layer.
            if (Beautypanel.layers.soft === document.activeLayer) {
                var confirmed = confirm(strings.confirm_unsharp_on_layer.split("$LAYER_NAME").join(Beautypanel.layers.soft.name));
                if (!confirmed) {
                    return;
                }
            }

            // ===================================================================== //
            // Open UnSharp Mask dialog
            var descriptor = new ActionDescriptor();

            descriptor.putUnitDouble(Codes.amount, Codes.percentUnit, amount);
            descriptor.putUnitDouble(Codes.radius, Codes.pixelsUnit, radius);
				descriptor.putInteger(Codes.threshold, threshold);
				
				//alert(app.activeDocument)

            returnValue = executeAction(Codes.unsharpMask, descriptor, dialogMode);

            var amount = returnValue.getDouble(Codes.amount);
            var radius = returnValue.getDouble(Codes.radius);
            var threshold = returnValue.getDouble(Codes.threshold);

            Beautypanel.log("Processed Command:unsharpMask document:'" + app.activeDocument.name + "'"
                + " Amount:" + amount
                + " Radius:" + radius
                + " Threshold:" + threshold);

        } catch (err) {
            alert("Error at Unsharp Mask: " + err);
        }

        return returnValue;
    },
    selectiveSharp: null,
    frequencySeparationSharp: null,
    maskedSharp: null,

    // Effects
    enhanceDetails: null,
    microContrast: null,
    ortonEffect: null,
    vignette: null,
    autumnEffect: null,

    // Save
    saveCopy: null

};

// ============ Extend $ ================ //

$.LayerUtils = Beautypanel.LayerUtils;
$.DocumentUtils = Beautypanel.DocumentUtils;
var layers = $.layers = Beautypanel.layers;



Beautypanel.log("Extension Beautypanel loaded");